import AppKit
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc


class TestNSAccessibilityProtocolsHelper(AppKit.NSObject):
    def accessibilityOrientation(self):
        return 1

    def setAccessibilityOrientation_(self, v):
        pass

    def accessibilityActivationPoint(self):
        return 1

    def accessibilityAttributedStringForRange_(self, v):
        return 1

    def accessibilityCellForColumn_row_(self, c, r):
        return 1

    def accessibilityColumnCount(self):
        return 1

    def accessibilityColumnIndexRange(self):
        return 1

    def accessibilityDisclosureLevel(self):
        return 1

    def accessibilityFrame(self):
        return 1

    def accessibilityFrameForRange_(self, v):
        return 1

    def accessibilityHorizontalUnits(self):
        return 1

    def accessibilityIndex(self):
        return 1

    def accessibilityInsertionPointLineNumber(self):
        return 1

    def accessibilityLabelValue(self):
        return 1

    def accessibilityLayoutPointForScreenPoint_(self, v):
        return 1

    def accessibilityLayoutSizeForScreenSize_(self, v):
        return 1

    def accessibilityLineForIndex_(self, v):
        return 1

    def accessibilityNumberOfCharacters(self):
        return 1

    def accessibilityPerformCancel(self):
        return 1

    def accessibilityPerformConfirm(self):
        return 1

    def accessibilityPerformDecrement(self):
        return 1

    def accessibilityPerformDelete(self):
        return 1

    def accessibilityPerformIncrement(self):
        return 1

    def accessibilityPerformPick(self):
        return 1

    def accessibilityPerformPress(self):
        return 1

    def accessibilityPerformRaise(self):
        return 1

    def accessibilityPerformShowAlternateUI(self):
        return 1

    def accessibilityPerformShowDefaultUI(self):
        return 1

    def accessibilityPerformShowMenu(self):
        return 1

    def accessibilityRTFForRange_(self, v):
        return 1

    def accessibilityRangeForIndex_(self, v):
        return 1

    def accessibilityRangeForLine_(self, v):
        return 1

    def accessibilityRangeForPosition_(self, v):
        return 1

    def accessibilityRowCount(self):
        return 1

    def accessibilityRowIndexRange(self):
        return 1

    def accessibilityRulerMarkerType(self):
        return 1

    def accessibilityScreenPointForLayoutPoint_(self, v):
        return 1

    def accessibilityScreenSizeForLayoutSize_(self, v):
        return 1

    def accessibilitySelectedTextRange(self):
        return 1

    def accessibilitySharedCharacterRange(self):
        return 1

    def accessibilitySortDirection(self):
        return 1

    def accessibilityStringForRange_(self, v):
        return 1

    def accessibilityStyleRangeForIndex_(self, v):
        return 1

    def accessibilityUnits(self):
        return 1

    def accessibilityVerticalUnits(self):
        return 1

    def accessibilityVisibleCharacterRange(self):
        return 1

    def isAccessibilityAlternateUIVisible(self):
        return 1

    def isAccessibilityDisclosed(self):
        return 1

    def setAccessibilityDisclosed_(self, v):
        pass

    def isAccessibilityEdited(self):
        return 1

    def isAccessibilityElement(self):
        return 1

    def isAccessibilityEnabled(self):
        return 1

    def isAccessibilityExpanded(self):
        return 1

    def isAccessibilityFocused(self):
        return 1

    def isAccessibilityFrontmost(self):
        return 1

    def isAccessibilityHidden(self):
        return 1

    def isAccessibilityMain(self):
        return 1

    def isAccessibilityMinimized(self):
        return 1

    def isAccessibilityModal(self):
        return 1

    def isAccessibilityOrderedByRow(self):
        return 1

    def isAccessibilityProtectedContent(self):
        return 1

    def isAccessibilitySelected(self):
        return 1

    def isAccessibilitySelectorAllowed_(self, sel):
        return 1

    def setAccessibilityActivationPoint_(self, p):
        pass

    def setAccessibilityAlternateUIVisible_(self, v):
        pass

    def setAccessibilityColumnCount_(self, v):
        pass

    def setAccessibilityColumnIndexRange_(self, v):
        pass

    def setAccessibilityDisclosureLevel_(self, v):
        pass

    def setAccessibilityEdited_(self, v):
        pass

    def setAccessibilityElement_(self, v):
        pass

    def setAccessibilityEnabled_(self, v):
        pass

    def setAccessibilityExpanded_(self, v):
        pass

    def setAccessibilityFocused_(self, v):
        pass

    def setAccessibilityFrame_(self, f):
        pass

    def setAccessibilityFrontmost_(self, v):
        pass

    def setAccessibilityHidden_(self, v):
        pass

    def setAccessibilityHorizontalUnits_(self, v):
        pass

    def setAccessibilityIndex_(self, v):
        pass

    def setAccessibilityInsertionPointLineNumber_(self, v):
        pass

    def setAccessibilityLabelValue_(self, v):
        pass

    def setAccessibilityMain_(self, v):
        return 1

    def setAccessibilityMinimized_(self, v):
        return 1

    def setAccessibilityModal_(self, v):
        return 1

    def setAccessibilityNumberOfCharacters_(self, v):
        pass

    def setAccessibilityOrderedByRow_(self, v):
        pass

    def setAccessibilityProtectedContent_(self, v):
        pass

    def setAccessibilityRowCount_(self, v):
        pass

    def setAccessibilityRowIndexRange_(self, v):
        pass

    def setAccessibilityRulerMarkerType_(self, v):
        pass

    def setAccessibilitySelectedTextRange_(self, v):
        pass

    def setAccessibilitySelected_(self, v):
        pass

    def setAccessibilitySharedCharacterRange_(self, v):
        pass

    def setAccessibilitySortDirection_(self, v):
        pass

    def setAccessibilityUnits_(self, v):
        pass

    def setAccessibilityVerticalUnits_(self, v):
        pass

    def setAccessibilityVisibleCharacterRange_(self, v):
        pass

    def isAccessibilityRequired(self):
        return 1


class TestNSAccessibilityProtocols(TestCase):
    @min_sdk_level("10.10")
    def testProtocolObjects(self):
        self.assertIsInstance(
            objc.protocolNamed("NSAccessibilityGroup"), objc.formal_protocol
        )
        self.assertIsInstance(
            objc.protocolNamed("NSAccessibilityElement"), objc.formal_protocol
        )
        self.assertIsInstance(
            objc.protocolNamed("NSAccessibilityButton"), objc.formal_protocol
        )
        self.assertIsInstance(
            objc.protocolNamed("NSAccessibilitySwitch"), objc.formal_protocol
        )
        self.assertIsInstance(
            objc.protocolNamed("NSAccessibilityRadioButton"), objc.formal_protocol
        )
        self.assertIsInstance(
            objc.protocolNamed("NSAccessibilityCheckBox"), objc.formal_protocol
        )
        self.assertIsInstance(
            objc.protocolNamed("NSAccessibilityStaticText"), objc.formal_protocol
        )
        self.assertIsInstance(
            objc.protocolNamed("NSAccessibilityNavigableStaticText"),
            objc.formal_protocol,
        )
        self.assertIsInstance(
            objc.protocolNamed("NSAccessibilityProgressIndicator"), objc.formal_protocol
        )
        self.assertIsInstance(
            objc.protocolNamed("NSAccessibilityStepper"), objc.formal_protocol
        )
        self.assertIsInstance(
            objc.protocolNamed("NSAccessibilitySlider"), objc.formal_protocol
        )
        self.assertIsInstance(
            objc.protocolNamed("NSAccessibilityImage"), objc.formal_protocol
        )
        self.assertIsInstance(
            objc.protocolNamed("NSAccessibilityContainsTransientUI"),
            objc.formal_protocol,
        )
        self.assertIsInstance(
            objc.protocolNamed("NSAccessibilityTable"), objc.formal_protocol
        )
        self.assertIsInstance(
            objc.protocolNamed("NSAccessibilityOutline"), objc.formal_protocol
        )
        self.assertIsInstance(
            objc.protocolNamed("NSAccessibilityList"), objc.formal_protocol
        )
        self.assertIsInstance(
            objc.protocolNamed("NSAccessibilityRow"), objc.formal_protocol
        )
        self.assertIsInstance(
            objc.protocolNamed("NSAccessibilityLayoutArea"), objc.formal_protocol
        )
        self.assertIsInstance(
            objc.protocolNamed("NSAccessibilityLayoutItem"), objc.formal_protocol
        )
        self.assertIsInstance(
            objc.protocolNamed("NSAccessibility"), objc.formal_protocol
        )

    @min_sdk_level("10.13")
    def testProtocolObjects10_13(self):
        self.assertIsInstance(
            objc.protocolNamed("NSAccessibilityElementLoading"), objc.formal_protocol
        )

    def testProtocols(self):
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.isAccessibilityFocused
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityFrame,
            AppKit.NSRect.__typestr__,
        )

        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformPress
        )

        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformIncrement
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformDecrement
        )

        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityVisibleCharacterRange,
            AppKit.NSRange.__typestr__,
        )

        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityFrameForRange_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityFrameForRange_,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityStringForRange_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityLineForIndex_,
            0,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityLineForIndex_,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityRangeForLine_,
            0,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityRangeForLine_,
            AppKit.NSRange.__typestr__,
        )

        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformIncrement
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformDecrement
        )

        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformIncrement
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformDecrement
        )

        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformIncrement
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformDecrement
        )

        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformShowAlternateUI
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformShowDefaultUI
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.isAccessibilityAlternateUIVisible
        )

        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityIndex, objc._C_NSInteger
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityDisclosureLevel,
            objc._C_NSInteger,
        )

        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.setAccessibilityFrame_,
            0,
            AppKit.NSRect.__typestr__,
        )

        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.isAccessibilityElement
        )
        self.assertArgIsBOOL(
            TestNSAccessibilityProtocolsHelper.setAccessibilityElement_, 0
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityFrame,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.setAccessibilityFrame_,
            0,
            AppKit.NSRect.__typestr__,
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.isAccessibilityFocused
        )
        self.assertArgIsBOOL(
            TestNSAccessibilityProtocolsHelper.setAccessibilityFocused_, 0
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityActivationPoint,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.setAccessibilityActivationPoint_,
            0,
            AppKit.NSPoint.__typestr__,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityOrientation,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.setAccessibilityOrientation_,
            0,
            objc._C_NSInteger,
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.isAccessibilitySelected
        )
        self.assertArgIsBOOL(
            TestNSAccessibilityProtocolsHelper.setAccessibilitySelected_, 0
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.isAccessibilityExpanded
        )
        self.assertArgIsBOOL(
            TestNSAccessibilityProtocolsHelper.setAccessibilityExpanded_, 0
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.isAccessibilityEdited
        )
        self.assertArgIsBOOL(
            TestNSAccessibilityProtocolsHelper.setAccessibilityEdited_, 0
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.isAccessibilityEnabled
        )
        self.assertArgIsBOOL(
            TestNSAccessibilityProtocolsHelper.setAccessibilityEnabled_, 0
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.isAccessibilityProtectedContent
        )
        self.assertArgIsBOOL(
            TestNSAccessibilityProtocolsHelper.setAccessibilityProtectedContent_, 0
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.isAccessibilityAlternateUIVisible
        )
        self.assertArgIsBOOL(
            TestNSAccessibilityProtocolsHelper.setAccessibilityAlternateUIVisible_, 0
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.isAccessibilityHidden
        )
        self.assertArgIsBOOL(
            TestNSAccessibilityProtocolsHelper.setAccessibilityHidden_, 0
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.isAccessibilityFrontmost
        )
        self.assertArgIsBOOL(
            TestNSAccessibilityProtocolsHelper.setAccessibilityFrontmost_, 0
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.isAccessibilityOrderedByRow
        )
        self.assertArgIsBOOL(
            TestNSAccessibilityProtocolsHelper.setAccessibilityOrderedByRow_, 0
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityHorizontalUnits,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.setAccessibilityHorizontalUnits_,
            0,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityVerticalUnits,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.setAccessibilityVerticalUnits_,
            0,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityLayoutPointForScreenPoint_,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityLayoutPointForScreenPoint_,
            0,
            AppKit.NSPoint.__typestr__,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityLayoutSizeForScreenSize_,
            AppKit.NSSize.__typestr__,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityLayoutSizeForScreenSize_,
            0,
            AppKit.NSSize.__typestr__,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityScreenPointForLayoutPoint_,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityScreenPointForLayoutPoint_,
            0,
            AppKit.NSPoint.__typestr__,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityScreenSizeForLayoutSize_,
            AppKit.NSSize.__typestr__,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityScreenSizeForLayoutSize_,
            0,
            AppKit.NSSize.__typestr__,
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.isAccessibilityDisclosed
        )
        self.assertArgIsBOOL(
            TestNSAccessibilityProtocolsHelper.setAccessibilityDisclosed_, 0
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityDisclosureLevel,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.setAccessibilityDisclosureLevel_,
            0,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityUnits, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.setAccessibilityUnits_,
            0,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityRulerMarkerType,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.setAccessibilityRulerMarkerType_,
            0,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityLabelValue, objc._C_FLT
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.setAccessibilityLabelValue_,
            0,
            objc._C_FLT,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityColumnCount,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.setAccessibilityColumnCount_,
            0,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityRowCount, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.setAccessibilityRowCount_,
            0,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityIndex, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.setAccessibilityIndex_,
            0,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilitySortDirection,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.setAccessibilitySortDirection_,
            0,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityCellForColumn_row_,
            0,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityCellForColumn_row_,
            1,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityRowIndexRange,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.setAccessibilityRowIndexRange_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityColumnIndexRange,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.setAccessibilityColumnIndexRange_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityInsertionPointLineNumber,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.setAccessibilityInsertionPointLineNumber_,
            0,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilitySharedCharacterRange,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.setAccessibilitySharedCharacterRange_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityVisibleCharacterRange,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.setAccessibilityVisibleCharacterRange_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityNumberOfCharacters,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.setAccessibilityNumberOfCharacters_,
            0,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilitySelectedTextRange,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.setAccessibilitySelectedTextRange_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityRangeForLine_,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityRangeForLine_,
            0,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityAttributedStringForRange_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityStringForRange_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityRangeForPosition_,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityRangeForPosition_,
            0,
            AppKit.NSPoint.__typestr__,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityRangeForIndex_,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityRangeForIndex_,
            0,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityFrameForRange_,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityFrameForRange_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityRTFForRange_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityStyleRangeForIndex_,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityStyleRangeForIndex_,
            0,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityStyleRangeForIndex_,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityStyleRangeForIndex_,
            0,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityLineForIndex_,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSAccessibilityProtocolsHelper.accessibilityLineForIndex_,
            0,
            objc._C_NSInteger,
        )
        self.assertResultIsBOOL(TestNSAccessibilityProtocolsHelper.isAccessibilityModal)
        self.assertArgIsBOOL(
            TestNSAccessibilityProtocolsHelper.setAccessibilityModal_, 0
        )
        self.assertResultIsBOOL(TestNSAccessibilityProtocolsHelper.isAccessibilityMain)
        self.assertArgIsBOOL(
            TestNSAccessibilityProtocolsHelper.setAccessibilityMain_, 0
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.isAccessibilityMinimized
        )
        self.assertArgIsBOOL(
            TestNSAccessibilityProtocolsHelper.setAccessibilityMinimized_, 0
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformCancel
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformConfirm
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformDecrement
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformDelete
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformIncrement
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformPick
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformPress
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformRaise
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformShowAlternateUI
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformShowDefaultUI
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.accessibilityPerformShowMenu
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.isAccessibilitySelectorAllowed_
        )
        self.assertResultIsBOOL(
            TestNSAccessibilityProtocolsHelper.isAccessibilityRequired
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc


class TestNSDrawerHelper(AppKit.NSObject):
    def drawerShouldOpen_(self, sender):
        return 1

    def drawerShouldClose_(self, sender):
        return 1

    def drawerWillResizeContents_toSize_(self, a, b):
        return 1


class TestNSDrawer(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSDrawerClosedState, 0)
        self.assertEqual(AppKit.NSDrawerOpeningState, 1)
        self.assertEqual(AppKit.NSDrawerOpenState, 2)
        self.assertEqual(AppKit.NSDrawerClosingState, 3)

        self.assertIsInstance(AppKit.NSDrawerWillOpenNotification, str)
        self.assertIsInstance(AppKit.NSDrawerDidOpenNotification, str)
        self.assertIsInstance(AppKit.NSDrawerWillCloseNotification, str)
        self.assertIsInstance(AppKit.NSDrawerDidCloseNotification, str)

    def testMethods(self):
        self.assertArgHasType(
            AppKit.NSDrawer.setMinContentSize_, 0, AppKit.NSSize.__typestr__
        )

    @min_sdk_level("10.10")
    def testProtocolObjects(self):
        objc.protocolNamed("NSDrawerDelegate")

    def testProtocols(self):
        self.assertResultIsBOOL(TestNSDrawerHelper.drawerShouldOpen_)
        self.assertResultIsBOOL(TestNSDrawerHelper.drawerShouldClose_)
        self.assertResultHasType(
            TestNSDrawerHelper.drawerWillResizeContents_toSize_,
            AppKit.NSSize.__typestr__,
        )
        self.assertArgHasType(
            TestNSDrawerHelper.drawerWillResizeContents_toSize_,
            1,
            AppKit.NSSize.__typestr__,
        )



import AppKit
from PyObjCTools.TestSupport import TestCase
import objc


class TestNSPageLayout(TestCase):
    def testMethods(self):
        self.assertArgIsSEL(
            AppKit.NSPageLayout.beginSheetWithPrintInfo_modalForWindow_delegate_didEndSelector_contextInfo_,  # noqa: B950
            3,
            b"v@:@" + objc._C_NSInteger + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSPageLayout.beginSheetWithPrintInfo_modalForWindow_delegate_didEndSelector_contextInfo_,  # noqa: B950
            4,
            b"^v",
        )



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestSubclassing(TestCase):
    def testBasicSubclassing(self):
        class NSObjectSubclass(Foundation.NSObject):
            def someRandomMethod(self):
                return 42

        subclassClass = Foundation.NSClassFromString("NSObjectSubclass")

        self.assertIsNot(subclassClass, None, "Failed to subclass NSObject.")

        subclassInstance = subclassClass.new()
        self.assertIsInstance(subclassInstance, subclassClass)
        self.assertIsInstance(subclassInstance, Foundation.NSObject)
        self.assertNotIsInstance(subclassInstance, Foundation.NSArray)

        subclassInstance.description()
        self.assertEqual(subclassInstance.someRandomMethod(), 42)

        self.assertIs(subclassInstance, subclassInstance, "Identity check failed.")
        self.assertIs(
            subclassInstance, subclassInstance.self(), "Identity check failed."
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSTextList(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSTextListPrependEnclosingMarker, 1)

    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertIsInstance(AppKit.NSTextListMarkerBox, str)
        self.assertIsInstance(AppKit.NSTextListMarkerCheck, str)
        self.assertIsInstance(AppKit.NSTextListMarkerCircle, str)
        self.assertIsInstance(AppKit.NSTextListMarkerDiamond, str)
        self.assertIsInstance(AppKit.NSTextListMarkerDisc, str)
        self.assertIsInstance(AppKit.NSTextListMarkerHyphen, str)
        self.assertIsInstance(AppKit.NSTextListMarkerSquare, str)
        self.assertIsInstance(AppKit.NSTextListMarkerLowercaseHexadecimal, str)
        self.assertIsInstance(AppKit.NSTextListMarkerUppercaseHexadecimal, str)
        self.assertIsInstance(AppKit.NSTextListMarkerOctal, str)
        self.assertIsInstance(AppKit.NSTextListMarkerLowercaseAlpha, str)
        self.assertIsInstance(AppKit.NSTextListMarkerUppercaseAlpha, str)
        self.assertIsInstance(AppKit.NSTextListMarkerLowercaseLatin, str)
        self.assertIsInstance(AppKit.NSTextListMarkerUppercaseLatin, str)
        self.assertIsInstance(AppKit.NSTextListMarkerLowercaseRoman, str)
        self.assertIsInstance(AppKit.NSTextListMarkerUppercaseRoman, str)
        self.assertIsInstance(AppKit.NSTextListMarkerDecimal, str)



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSCachedImageRep(TestCase):
    def testMethods(self):
        self.assertArgIsBOOL(
            AppKit.NSCachedImageRep.initWithSize_depth_separate_alpha_, 2
        )
        self.assertArgIsBOOL(
            AppKit.NSCachedImageRep.initWithSize_depth_separate_alpha_, 3
        )



import os
from threading import Thread

import AppKit
import Foundation
from Foundation import NSAutoreleasePool, NSLog, NSObject
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestRegr(TestCase):
    def testFSRepr(self):
        fm = Foundation.NSFileManager.defaultManager()
        self.assertRaises(
            TypeError, fm.stringWithFileSystemRepresentation_length_, b"/var"
        )
        self.assertEqual(
            "/var", fm.stringWithFileSystemRepresentation_length_(b"/var/boo", 4)
        )

    def testThreadHang(self):

        # Temporarily redirect stderr to a file, this allows us to check
        # that NSLog actually wrote some text.
        fp = os.open("/tmp/pyobjc-thread.txt", os.O_RDWR | os.O_CREAT, 0o666)
        dupped = os.dup(2)
        os.dup2(fp, 2)

        try:

            class ThreadHangObject(NSObject):
                def init(self):
                    self.t = MyThread()
                    self.t.start()
                    return self

            aList = []

            class MyThread(Thread):
                def run(self):
                    pool = NSAutoreleasePool.alloc().init()  # noqa: F841
                    aList.append("before")
                    NSLog("does this print?")
                    aList.append("after")

            o = ThreadHangObject.alloc().init()
            o.t.join()

        finally:
            os.close(fp)
            os.dup2(dupped, 2)

        with open("/tmp/pyobjc-thread.txt") as fp:
            data = fp.read()
        self.assertTrue("does this print?" in data)

        self.assertEqual(aList, ["before", "after"])

    def testMemoryInit(self):
        """
        Regression test for bug #814683, that didn't initialize the memory
        space for output parameters correctly.
        """
        if not hasattr(Foundation, "NSPropertyListSerialization"):
            return

        plist = 0

        r = Foundation.NSPropertyListSerialization.dataFromPropertyList_format_errorDescription_(  # noqa: B950
            plist, Foundation.NSPropertyListXMLFormat_v1_0, None
        )
        self.assertEqual(r[1], None)
        r = Foundation.NSPropertyListSerialization.dataFromPropertyList_format_errorDescription_(  # noqa: B950
            plist, Foundation.NSPropertyListXMLFormat_v1_0, None
        )
        self.assertEqual(r[1], None)

    def testTypeOverrideProblem(self):
        """
        A bug in the medatata machinery caused a crash.
        """
        cls = AppKit.NSOpenGLPixelFormat
        dir(cls)

        _ = cls.alloc().initWithAttributes_(
            (
                AppKit.NSOpenGLPFAAccelerated,
                AppKit.NSOpenGLPFANoRecovery,
                AppKit.NSOpenGLPFAColorSize,
                32,
            )
        )

    @min_os_level("10.6")
    def testBinaryPlist(self):
        for pl in ({"key": 2 ** 64 - 1}, {"key": 2 ** 16 - 1}):
            with self.subTest(pl):
                (
                    data,
                    error,
                ) = Foundation.NSPropertyListSerialization.dataWithPropertyList_format_options_error_(  # noqa: B950
                    pl, Foundation.NSPropertyListBinaryFormat_v1_0, 0, None
                )
            (
                restored,
                plformat,
                error,
            ) = Foundation.NSPropertyListSerialization.propertyListWithData_options_format_error_(  # noqa: B950
                data, 0, None, None
            )
            self.assertEqual(pl, restored)



import Cocoa
from objc import super


class CaseInsensitivePredicateTemplate(Cocoa.NSPredicateEditorRowTemplate):
    def predicateWithSubpredicates_(self, subpredicates):
        # we only make NSComparisonPredicates
        predicate = super().predicateWithSubpredicates_(subpredicates)

        # construct an identical predicate, but add the
        # NSCaseInsensitivePredicateOption flag
        return Cocoa.NSComparisonPredicate.predicateWithLeftExpression_rightExpression_modifier_type_options_(  # noqa: B950
            predicate.leftExpression(),
            predicate.rightExpression(),
            predicate.comparisonPredicateModifier(),
            predicate.predicateOperatorType(),
            predicate.options() | Cocoa.NSCaseInsensitivePredicateOption,
        )



import Foundation
import CoreFoundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSPropertyList(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(
            Foundation.NSPropertyListSerialization.propertyList_isValidForFormat_
        )
        self.assertArgIsOut(
            Foundation.NSPropertyListSerialization.dataFromPropertyList_format_errorDescription_,  # noqa: B950
            2,
        )
        self.assertArgIsOut(
            Foundation.NSPropertyListSerialization.propertyListFromData_mutabilityOption_format_errorDescription_,  # noqa: B950
            3,
        )

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertArgIsOut(
            Foundation.NSPropertyListSerialization.dataWithPropertyList_format_options_error_,
            3,
        )
        self.assertArgIsOut(
            Foundation.NSPropertyListSerialization.writePropertyList_toStream_format_options_error_,  # noqa: B950
            4,
        )
        self.assertArgIsOut(
            Foundation.NSPropertyListSerialization.propertyListWithData_options_format_error_,
            3,
        )
        self.assertArgIsOut(
            Foundation.NSPropertyListSerialization.propertyListWithStream_options_format_error_,
            3,
        )

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(
            Foundation.NSPropertyListImmutable, CoreFoundation.kCFPropertyListImmutable
        )
        self.assertEqual(
            Foundation.NSPropertyListMutableContainers,
            CoreFoundation.kCFPropertyListMutableContainers,
        )
        self.assertEqual(
            Foundation.NSPropertyListMutableContainersAndLeaves,
            CoreFoundation.kCFPropertyListMutableContainersAndLeaves,
        )

        self.assertEqual(
            Foundation.NSPropertyListOpenStepFormat,
            CoreFoundation.kCFPropertyListOpenStepFormat,
        )
        self.assertEqual(
            Foundation.NSPropertyListXMLFormat_v1_0,
            CoreFoundation.kCFPropertyListXMLFormat_v1_0,
        )
        self.assertEqual(
            Foundation.NSPropertyListBinaryFormat_v1_0,
            CoreFoundation.kCFPropertyListBinaryFormat_v1_0,
        )



import Foundation
import CoreFoundation
from PyObjCTools.TestSupport import TestCase


class TestNSByteOrder(TestCase):
    def testConstants(self):
        self.assertEqual(
            Foundation.NS_UnknownByteOrder, CoreFoundation.CFByteOrderUnknown
        )
        self.assertEqual(
            Foundation.NS_LittleEndian, CoreFoundation.CFByteOrderLittleEndian
        )
        self.assertEqual(Foundation.NS_BigEndian, CoreFoundation.CFByteOrderBigEndian)

    def testInlines(self):
        self.assertEqual(
            Foundation.NSHostByteOrder(), CoreFoundation.CFByteOrderGetCurrent()
        )
        self.assertEqual(Foundation.NSSwapShort(350), CoreFoundation.CFSwapInt16(350))
        self.assertEqual(Foundation.NSSwapInt(350), CoreFoundation.CFSwapInt32(350))

        self.assertEqual(Foundation.NSSwapLong(350), CoreFoundation.CFSwapInt64(350))
        self.assertEqual(
            Foundation.NSSwapBigLongToHost(350),
            CoreFoundation.CFSwapInt64BigToHost(350),
        )
        self.assertEqual(
            Foundation.NSSwapHostLongToBig(350),
            CoreFoundation.CFSwapInt64HostToBig(350),
        )
        self.assertEqual(
            Foundation.NSSwapLittleLongToHost(350),
            CoreFoundation.CFSwapInt64LittleToHost(350),
        )
        self.assertEqual(
            Foundation.NSSwapHostLongToLittle(350),
            CoreFoundation.CFSwapInt64HostToLittle(350),
        )

        self.assertEqual(
            Foundation.NSSwapLongLong(350), CoreFoundation.CFSwapInt64(350)
        )
        self.assertEqual(
            Foundation.NSSwapBigShortToHost(350),
            CoreFoundation.CFSwapInt16BigToHost(350),
        )
        self.assertEqual(
            Foundation.NSSwapBigIntToHost(350), CoreFoundation.CFSwapInt32BigToHost(350)
        )
        self.assertEqual(
            Foundation.NSSwapBigLongLongToHost(350),
            CoreFoundation.CFSwapInt64BigToHost(350),
        )
        self.assertEqual(
            Foundation.NSSwapHostShortToBig(350),
            CoreFoundation.CFSwapInt16HostToBig(350),
        )
        self.assertEqual(
            Foundation.NSSwapHostIntToBig(350), CoreFoundation.CFSwapInt32HostToBig(350)
        )
        self.assertEqual(
            Foundation.NSSwapHostLongLongToBig(350),
            CoreFoundation.CFSwapInt64HostToBig(350),
        )
        self.assertEqual(
            Foundation.NSSwapLittleShortToHost(350),
            CoreFoundation.CFSwapInt16LittleToHost(350),
        )
        self.assertEqual(
            Foundation.NSSwapLittleIntToHost(350),
            CoreFoundation.CFSwapInt32LittleToHost(350),
        )
        self.assertEqual(
            Foundation.NSSwapLittleLongLongToHost(350),
            CoreFoundation.CFSwapInt64LittleToHost(350),
        )
        self.assertEqual(
            Foundation.NSSwapHostShortToLittle(350),
            CoreFoundation.CFSwapInt16HostToLittle(350),
        )
        self.assertEqual(
            Foundation.NSSwapHostIntToLittle(350),
            CoreFoundation.CFSwapInt32HostToLittle(350),
        )
        self.assertEqual(
            Foundation.NSSwapHostLongLongToLittle(350),
            CoreFoundation.CFSwapInt64HostToLittle(350),
        )

        v = Foundation.NSConvertHostFloatToSwapped(55.0)
        self.assertIsInstance(v, Foundation.NSSwappedFloat)
        n = Foundation.NSConvertSwappedFloatToHost(v)
        self.assertEqual(n, 55.0)

        v = Foundation.NSSwapFloat(v)
        self.assertIsInstance(v, Foundation.NSSwappedFloat)
        v = Foundation.NSConvertHostDoubleToSwapped(55.0)
        self.assertIsInstance(v, Foundation.NSSwappedDouble)
        n = Foundation.NSConvertSwappedDoubleToHost(v)
        self.assertEqual(n, 55.0)

        v = Foundation.NSSwapDouble(v)
        self.assertIsInstance(v, Foundation.NSSwappedDouble)
        n = Foundation.NSSwapBigDoubleToHost(v)
        self.assertIsInstance(n, float)
        n = Foundation.NSSwapLittleDoubleToHost(v)
        self.assertIsInstance(n, float)
        v = Foundation.NSConvertHostFloatToSwapped(55.0)
        n = Foundation.NSSwapBigFloatToHost(v)
        self.assertIsInstance(n, float)
        n = Foundation.NSSwapLittleFloatToHost(v)
        self.assertIsInstance(n, float)
        v = Foundation.NSSwapHostDoubleToBig(55.0)
        self.assertIsInstance(v, Foundation.NSSwappedDouble)
        v = Foundation.NSSwapHostDoubleToLittle(55.0)
        self.assertIsInstance(v, Foundation.NSSwappedDouble)
        v = Foundation.NSSwapHostFloatToBig(55.0)
        self.assertIsInstance(v, Foundation.NSSwappedFloat)
        v = Foundation.NSSwapHostFloatToLittle(55.0)
        self.assertIsInstance(v, Foundation.NSSwappedFloat)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSURLProtectionSpace(TestCase):
    def testConstants(self):
        self.assertIsInstance(Foundation.NSURLProtectionSpaceHTTPProxy, str)
        self.assertIsInstance(Foundation.NSURLProtectionSpaceHTTPSProxy, str)
        self.assertIsInstance(Foundation.NSURLProtectionSpaceFTPProxy, str)
        self.assertIsInstance(Foundation.NSURLProtectionSpaceSOCKSProxy, str)
        self.assertIsInstance(Foundation.NSURLAuthenticationMethodDefault, str)
        self.assertIsInstance(Foundation.NSURLAuthenticationMethodHTTPBasic, str)
        self.assertIsInstance(Foundation.NSURLAuthenticationMethodHTTPDigest, str)
        self.assertIsInstance(Foundation.NSURLAuthenticationMethodHTMLForm, str)

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertIsInstance(Foundation.NSURLProtectionSpaceHTTP, str)
        self.assertIsInstance(Foundation.NSURLProtectionSpaceHTTPS, str)
        self.assertIsInstance(Foundation.NSURLProtectionSpaceFTP, str)
        self.assertIsInstance(Foundation.NSURLAuthenticationMethodNTLM, str)
        self.assertIsInstance(Foundation.NSURLAuthenticationMethodNegotiate, str)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertIsInstance(
            Foundation.NSURLAuthenticationMethodClientCertificate, str
        )
        self.assertIsInstance(Foundation.NSURLAuthenticationMethodServerTrust, str)

    def testMethods(self):
        self.assertResultIsBOOL(
            Foundation.NSURLProtectionSpace.receivesCredentialSecurely
        )
        self.assertResultIsBOOL(Foundation.NSURLProtectionSpace.isProxy)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSOutlineViewHelper(AppKit.NSObject):
    def outlineView_sizeToFitWidthOfColumn_(self, v, c):
        return 1

    def outlineView_shouldReorderColumn_toColumn_(self, v, c1, c2):
        return 1

    def outlineView_shouldShowOutlineCellForItem_(self, v, i):
        return 1

    def outlineView_child_ofItem_(self, ov, nr, item):
        return 1

    def outlineView_isItemExpandable_(self, ov, item):
        return 1

    def outlineView_numberOfChildrenOfItem_(self, ov, item):
        return 1

    def outlineView_objectValueForTableColumn_byItem_(self, ov, tc, item):
        return 1

    def outlineView_setObjectValue_forTableColumn_byItem_(self, ov, value, tc, item):
        pass

    def outlineView_itemForPersistentObject_(self, ov, po):
        return 1

    def outlineView_persistentObjectForItem_(self, ov, item):
        return 1

    def outlineView_sortDescriptorsDidChange_(self, ov, old):
        pass

    def outlineView_writeItems_toPasteboard_(self, ov, items, pb):
        return 1

    def outlineView_validateDrop_proposedItem_proposedChildIndex_(
        self, ov, dr, item, idx
    ):
        return 1

    def outlineView_acceptDrop_item_childIndex_(self, ov, dr, it, idx):
        return 1

    def outlineView_namesOfPromisedFilesDroppedAtDestination_forDraggedItems_(
        self, ov, dr, it
    ):
        return 1

    def outlineView_willDisplayCell_forTableColumn_item_(self, ov, c, tc, i):
        pass

    def outlineView_shouldEditTableColumn_item_(self, ov, tc, i):
        return 1

    def selectionShouldChangeInOutlineView_(self, ov):
        return 1

    def outlineView_selectionIndexesForProposedSelection_(self, ov, idx):
        return 1

    def outlineView_shouldSelectItem_(self, ov, tc):
        return 1

    def outlineView_shouldSelectTableColumn_(self, ov, tc):
        return 1

    def outlineView_toolTipForCell_rect_tableColumn_item_mouseLocation_(
        self, ov, c, r, tc, it, ml
    ):
        return 1

    def outlineView_heightOfRowByItem_(self, ov, item):
        return 1

    def outlineView_typeSelectStringForTableColumn_item_(self, ov, tc, item):
        return 1

    def outlineView_nextTypeSelectMatchFromItem_toItem_forString_(self, ov, si, ei, ss):
        return 1

    def outlineView_shouldTypeSelectForEvent_withCurrentSearchString_(self, ov, ev, ss):
        return 1

    def outlineView_shouldShowCellExpansionForTableColumn_item_(self, ov, tc, it):
        return 1

    def outlineView_shouldTrackCell_forTableColumn_item_(self, ov, c, tc, it):
        return 1

    def outlineView_dataCellForTableColumn_item_(self, ov, tc, it):
        return 1

    def outlineView_isGroupItem_(self, ov, item):
        return 1

    def outlineView_shouldExpandItem_(self, ov, it):
        return 1

    def outlineView_shouldCollapseItem_(self, ov, it):
        return 1

    def outlineView_willDisplayOutlineCell_forTableColumn_item_(self, ov, c, tc, i):
        pass

    def outlineView_draggingSession_willBeginAtPoint_(self, a, b, c):
        pass

    def outlineView_draggingSession_endedAtPoint_(self, a, b, c):
        pass

    def outlineView_didAddRowView_forRow_(self, a, b, c):
        pass

    def outlineView_didRemoveRowView_forRow_(self, a, b, c):
        pass


class TestNSOutlineView(TestCase):
    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertIsInstance(AppKit.NSOutlineViewDisclosureButtonKey, str)
        self.assertIsInstance(AppKit.NSOutlineViewShowHideButtonKey, str)

    def testConstants(self):
        self.assertEqual(AppKit.NSOutlineViewDropOnItemIndex, -1)

        self.assertIsInstance(AppKit.NSOutlineViewSelectionDidChangeNotification, str)
        self.assertIsInstance(AppKit.NSOutlineViewColumnDidMoveNotification, str)
        self.assertIsInstance(AppKit.NSOutlineViewColumnDidResizeNotification, str)
        self.assertIsInstance(AppKit.NSOutlineViewSelectionIsChangingNotification, str)
        self.assertIsInstance(AppKit.NSOutlineViewItemWillExpandNotification, str)
        self.assertIsInstance(AppKit.NSOutlineViewItemDidExpandNotification, str)
        self.assertIsInstance(AppKit.NSOutlineViewItemWillCollapseNotification, str)
        self.assertIsInstance(AppKit.NSOutlineViewItemDidCollapseNotification, str)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSOutlineView.isExpandable_)
        self.assertArgIsBOOL(AppKit.NSOutlineView.expandItem_expandChildren_, 1)
        self.assertArgIsBOOL(AppKit.NSOutlineView.collapseItem_collapseChildren_, 1)
        self.assertArgIsBOOL(AppKit.NSOutlineView.reloadItem_reloadChildren_, 1)
        self.assertResultIsBOOL(AppKit.NSOutlineView.isItemExpanded_)
        self.assertResultIsBOOL(AppKit.NSOutlineView.indentationMarkerFollowsCell)
        self.assertArgIsBOOL(AppKit.NSOutlineView.setIndentationMarkerFollowsCell_, 0)
        self.assertResultIsBOOL(AppKit.NSOutlineView.autoresizesOutlineColumn)
        self.assertArgIsBOOL(AppKit.NSOutlineView.setAutoresizesOutlineColumn_, 0)
        self.assertResultIsBOOL(
            AppKit.NSOutlineView.shouldCollapseAutoExpandedItemsForDeposited_
        )
        self.assertArgIsBOOL(
            AppKit.NSOutlineView.shouldCollapseAutoExpandedItemsForDeposited_, 0
        )
        self.assertResultIsBOOL(AppKit.NSOutlineView.autosaveExpandedItems)
        self.assertArgIsBOOL(AppKit.NSOutlineView.setAutosaveExpandedItems_, 0)

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(AppKit.NSOutlineView.stronglyReferencesItems)
        self.assertArgIsBOOL(AppKit.NSOutlineView.setStronglyReferencesItems_, 0)

    @min_sdk_level("10.6")
    def testProtocolObjects(self):
        objc.protocolNamed("NSOutlineViewDelegate")
        objc.protocolNamed("NSOutlineViewDataSource")

    def testProtocols(self):
        self.assertArgHasType(
            TestNSOutlineViewHelper.outlineView_child_ofItem_, 1, objc._C_NSInteger
        )
        self.assertResultIsBOOL(TestNSOutlineViewHelper.outlineView_isItemExpandable_)
        self.assertResultHasType(
            TestNSOutlineViewHelper.outlineView_numberOfChildrenOfItem_,
            objc._C_NSInteger,
        )
        self.assertResultIsBOOL(
            TestNSOutlineViewHelper.outlineView_writeItems_toPasteboard_
        )
        self.assertResultHasType(
            TestNSOutlineViewHelper.outlineView_validateDrop_proposedItem_proposedChildIndex_,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSOutlineViewHelper.outlineView_validateDrop_proposedItem_proposedChildIndex_,
            3,
            objc._C_NSInteger,
        )
        self.assertResultIsBOOL(
            TestNSOutlineViewHelper.outlineView_acceptDrop_item_childIndex_
        )
        self.assertResultIsBOOL(
            TestNSOutlineViewHelper.outlineView_shouldEditTableColumn_item_
        )
        self.assertResultIsBOOL(
            TestNSOutlineViewHelper.selectionShouldChangeInOutlineView_
        )
        self.assertResultIsBOOL(TestNSOutlineViewHelper.outlineView_shouldSelectItem_)
        self.assertResultIsBOOL(
            TestNSOutlineViewHelper.outlineView_shouldSelectTableColumn_
        )
        self.assertArgHasType(
            TestNSOutlineViewHelper.outlineView_toolTipForCell_rect_tableColumn_item_mouseLocation_,  # noqa: B950
            2,
            b"N^" + AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSOutlineViewHelper.outlineView_toolTipForCell_rect_tableColumn_item_mouseLocation_,  # noqa: B950
            5,
            AppKit.NSPoint.__typestr__,
        )
        self.assertResultHasType(
            TestNSOutlineViewHelper.outlineView_heightOfRowByItem_, objc._C_CGFloat
        )
        self.assertResultIsBOOL(TestNSOutlineViewHelper.outlineView_shouldExpandItem_)
        self.assertResultIsBOOL(TestNSOutlineViewHelper.outlineView_shouldCollapseItem_)

        self.assertArgHasType(
            TestNSOutlineViewHelper.outlineView_didAddRowView_forRow_,
            2,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSOutlineViewHelper.outlineView_didRemoveRowView_forRow_,
            2,
            objc._C_NSInteger,
        )

    @min_os_level("10.5")
    def testProtocols10_5(self):
        self.assertResultIsBOOL(
            TestNSOutlineViewHelper.outlineView_shouldTypeSelectForEvent_withCurrentSearchString_  # noqa: B950
        )
        self.assertResultIsBOOL(
            TestNSOutlineViewHelper.outlineView_shouldShowCellExpansionForTableColumn_item_
        )
        self.assertResultIsBOOL(
            TestNSOutlineViewHelper.outlineView_shouldTrackCell_forTableColumn_item_
        )
        self.assertResultIsBOOL(TestNSOutlineViewHelper.outlineView_isGroupItem_)

    @min_os_level("10.6")
    def testProtocols10_6(self):
        self.assertResultHasType(
            TestNSOutlineViewHelper.outlineView_sizeToFitWidthOfColumn_, objc._C_CGFloat
        )
        self.assertArgHasType(
            TestNSOutlineViewHelper.outlineView_sizeToFitWidthOfColumn_,
            1,
            objc._C_NSInteger,
        )
        self.assertResultIsBOOL(
            TestNSOutlineViewHelper.outlineView_shouldReorderColumn_toColumn_
        )
        self.assertArgHasType(
            TestNSOutlineViewHelper.outlineView_shouldReorderColumn_toColumn_,
            1,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSOutlineViewHelper.outlineView_shouldReorderColumn_toColumn_,
            2,
            objc._C_NSInteger,
        )

        self.assertResultIsBOOL(
            TestNSOutlineViewHelper.outlineView_shouldShowOutlineCellForItem_
        )

    @min_os_level("10.7")
    def testProtocols10_7(self):
        self.assertArgHasType(
            TestNSOutlineViewHelper.outlineView_draggingSession_willBeginAtPoint_,
            2,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestNSOutlineViewHelper.outlineView_draggingSession_endedAtPoint_,
            2,
            AppKit.NSPoint.__typestr__,
        )



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure


class TestNSMapTable(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSMapTableStrongMemory, 0)
        self.assertEqual(
            Foundation.NSMapTableZeroingWeakMemory,
            Foundation.NSPointerFunctionsZeroingWeakMemory,
        )
        self.assertEqual(
            Foundation.NSMapTableCopyIn, Foundation.NSPointerFunctionsCopyIn
        )
        self.assertEqual(
            Foundation.NSMapTableObjectPointerPersonality,
            Foundation.NSPointerFunctionsObjectPointerPersonality,
        )

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertEqual(
            Foundation.NSMapTableWeakMemory, Foundation.NSPointerFunctionsWeakMemory
        )

    @expectedFailure
    def testFunctions(self):
        self.fail("NSMapTable C-API is untested")



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSURLConnectionHelper(Foundation.NSObject):
    def connection_canAuthenticateAgainstProtectionSpace_(self, a, b):
        return 1

    def connectionShouldUseCredentialStorage_(self, a):
        return 1

    def connection_didSendBodyData_totalBytesWritten_totalBytesExpectedToWrite_(
        self, a, b, c, d
    ):
        return 1

    def connection_didWriteData_totalBytesWritten_expectedTotalBytes_(self, a, b, c, d):
        return 1

    def connectionDidResumeDownloading_totalBytesWritten_expectedTotalBytes_(
        self, a, b, c
    ):
        return 1


class TestNSURLConnection(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSURLConnection.canHandleRequest_)

        self.assertArgIsBOOL(
            Foundation.NSURLConnection.initWithRequest_delegate_startImmediately_, 2
        )

        self.assertArgIsOut(
            Foundation.NSURLConnection.sendSynchronousRequest_returningResponse_error_,
            1,
        )
        self.assertArgIsOut(
            Foundation.NSURLConnection.sendSynchronousRequest_returningResponse_error_,
            2,
        )

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultIsBOOL(
            TestNSURLConnectionHelper.connection_canAuthenticateAgainstProtectionSpace_
        )
        self.assertResultIsBOOL(
            TestNSURLConnectionHelper.connectionShouldUseCredentialStorage_
        )
        self.assertArgHasType(
            TestNSURLConnectionHelper.connection_didSendBodyData_totalBytesWritten_totalBytesExpectedToWrite_,  # noqa: B950
            1,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSURLConnectionHelper.connection_didSendBodyData_totalBytesWritten_totalBytesExpectedToWrite_,  # noqa: B950
            2,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSURLConnectionHelper.connection_didSendBodyData_totalBytesWritten_totalBytesExpectedToWrite_,  # noqa: B950
            3,
            objc._C_NSInteger,
        )

        self.assertArgHasType(
            TestNSURLConnectionHelper.connection_didWriteData_totalBytesWritten_expectedTotalBytes_,  # noqa: B950
            1,
            objc._C_LNG_LNG,
        )
        self.assertArgHasType(
            TestNSURLConnectionHelper.connection_didWriteData_totalBytesWritten_expectedTotalBytes_,  # noqa: B950
            2,
            objc._C_LNG_LNG,
        )
        self.assertArgHasType(
            TestNSURLConnectionHelper.connection_didWriteData_totalBytesWritten_expectedTotalBytes_,  # noqa: B950
            3,
            objc._C_LNG_LNG,
        )

        self.assertArgHasType(
            TestNSURLConnectionHelper.connectionDidResumeDownloading_totalBytesWritten_expectedTotalBytes_,  # noqa: B950
            1,
            objc._C_LNG_LNG,
        )
        self.assertArgHasType(
            TestNSURLConnectionHelper.connectionDidResumeDownloading_totalBytesWritten_expectedTotalBytes_,  # noqa: B950
            2,
            objc._C_LNG_LNG,
        )

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertArgIsBlock(
            Foundation.NSURLConnection.sendAsynchronousRequest_queue_completionHandler_,
            2,
            b"v@@",
        )

    @min_sdk_level("10.7")
    def testProtocolObjects(self):
        objc.protocolNamed("NSURLConnectionDelegate")

    @min_sdk_level("10.10")
    def testProtocolObjects10_10(self):
        objc.protocolNamed("NSURLConnectionDataDelegate")
        objc.protocolNamed("NSURLConnectionDownloadDelegate")



# NOTE: This file only contains basic tests of the keyvalue coding header definitions,
# test_keyvalue contains tests for the actualy KVC/KVO mechanisms.
import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSKeyValueCodingHelper(Foundation.NSObject):
    def validateValue_forKey_error_(self, a, b, c):
        return 1

    def validateValue_forKeyPath_error_(self, a, b, c):
        return 1

    def useStoredAccessor(self):
        return 1


class TestNSKeyValueCoding(TestCase):
    def testConstants(self):
        self.assertIsInstance(Foundation.NSUndefinedKeyException, str)
        self.assertIsInstance(Foundation.NSAverageKeyValueOperator, str)
        self.assertIsInstance(Foundation.NSCountKeyValueOperator, str)
        self.assertIsInstance(Foundation.NSDistinctUnionOfArraysKeyValueOperator, str)
        self.assertIsInstance(Foundation.NSDistinctUnionOfObjectsKeyValueOperator, str)
        self.assertIsInstance(Foundation.NSDistinctUnionOfSetsKeyValueOperator, str)
        self.assertIsInstance(Foundation.NSMaximumKeyValueOperator, str)
        self.assertIsInstance(Foundation.NSMinimumKeyValueOperator, str)
        self.assertIsInstance(Foundation.NSSumKeyValueOperator, str)
        self.assertIsInstance(Foundation.NSUnionOfArraysKeyValueOperator, str)
        self.assertIsInstance(Foundation.NSUnionOfObjectsKeyValueOperator, str)
        self.assertIsInstance(Foundation.NSUnionOfSetsKeyValueOperator, str)

    def testDefineValidation(self):
        o = Foundation.NSObject.alloc().init()

        m = o.validateValue_forKey_error_.__metadata__()
        self.assertEqual(m["arguments"][4]["type"], b"o^@")

        m = o.validateValue_forKeyPath_error_.__metadata__()
        self.assertEqual(m["arguments"][4]["type"], b"o^@")

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSObject.accessInstanceVariablesDirectly)

        self.assertResultIsBOOL(TestNSKeyValueCodingHelper.validateValue_forKey_error_)
        self.assertArgIsOut(TestNSKeyValueCodingHelper.validateValue_forKey_error_, 2)
        self.assertResultIsBOOL(
            TestNSKeyValueCodingHelper.validateValue_forKeyPath_error_
        )

        self.assertArgIsInOut(
            TestNSKeyValueCodingHelper.validateValue_forKeyPath_error_, 0
        )
        self.assertArgIsOut(
            TestNSKeyValueCodingHelper.validateValue_forKeyPath_error_, 2
        )
        self.assertResultIsBOOL(TestNSKeyValueCodingHelper.useStoredAccessor)



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSCalendarDate(TestCase):
    def testOutput(self):
        obj = Foundation.NSCalendarDate.dateWithYear_month_day_hour_minute_second_timeZone_(
            2008, 12, 5, 14, 15, 16, Foundation.NSTimeZone.systemTimeZone()
        )
        obj2 = Foundation.NSCalendarDate.dateWithYear_month_day_hour_minute_second_timeZone_(
            2007, 9, 8, 19, 12, 10, Foundation.NSTimeZone.systemTimeZone()
        )

        m = obj.years_months_days_hours_minutes_seconds_sinceDate_
        self.assertArgIsOut(m, 0)
        self.assertArgIsOut(m, 1)
        self.assertArgIsOut(m, 2)
        self.assertArgIsOut(m, 3)
        self.assertArgIsOut(m, 4)
        self.assertArgIsOut(m, 5)

        (
            years,
            months,
            days,
            hours,
            minutes,
            seconds,
        ) = obj.years_months_days_hours_minutes_seconds_sinceDate_(
            None, None, None, None, None, None, obj2
        )
        self.assertEqual(years, 1)
        self.assertEqual(months, 2)
        self.assertEqual(days, 26)
        self.assertEqual(hours, 19)
        self.assertEqual(minutes, 3)
        self.assertEqual(seconds, 6)



import AppKit  # noqa: F401
import objc
from PyObjCTools.TestSupport import TestCase, min_sdk_level, min_os_level


class TestProtocolsExisting(TestCase):
    @min_sdk_level("10.10")
    def testProtocols10_10(self):
        objc.protocolNamed("NSAlertDelegate")
        objc.protocolNamed("NSApplicationDelegate")
        objc.protocolNamed("NSCollectionViewDelegate")
        objc.protocolNamed("NSDatePickerCellDelegate")
        objc.protocolNamed("NSDockTilePlugIn")
        objc.protocolNamed("NSDrawerDelegate")
        objc.protocolNamed("NSImageDelegate")
        objc.protocolNamed("NSPageControllerDelegate")
        objc.protocolNamed("NSPathCellDelegate")
        objc.protocolNamed("NSSoundDelegate")
        objc.protocolNamed("NSSpeechRecognizerDelegate")
        objc.protocolNamed("NSSpeechSynthesizerDelegate")
        objc.protocolNamed("NSTextFinderClient")
        objc.protocolNamed("NSTextStorageDelegate")
        objc.protocolNamed("NSTokenFieldCellDelegate")
        objc.protocolNamed("NSTokenFieldDelegate")
        objc.protocolNamed("NSUserInterfaceItemSearching")
        objc.protocolNamed("NSComboBoxCellDataSource")
        objc.protocolNamed("NSSplitViewDelegate")
        objc.protocolNamed("NSTabViewDelegate")

    @min_os_level("10.6")
    def testProtocols10_6(self):
        objc.protocolNamed("NSAnimatablePropertyContainer")
        objc.protocolNamed("NSAnimationDelegate")
        objc.protocolNamed("NSBrowserDelegate")
        objc.protocolNamed("NSChangeSpelling")
        objc.protocolNamed("NSColorPickingCustom")
        objc.protocolNamed("NSColorPickingDefault")
        objc.protocolNamed("NSComboBoxDataSource")
        objc.protocolNamed("NSComboBoxDelegate")
        objc.protocolNamed("NSControlTextEditingDelegate")
        objc.protocolNamed("NSDraggingInfo")
        objc.protocolNamed("NSGlyphStorage")
        objc.protocolNamed("NSIgnoreMisspelledWords")
        objc.protocolNamed("NSInputServerMouseTracker")
        objc.protocolNamed("NSInputServiceProvider")
        objc.protocolNamed("NSLayoutManagerDelegate")
        objc.protocolNamed("NSMatrixDelegate")
        objc.protocolNamed("NSMenuDelegate")
        objc.protocolNamed("NSOpenSavePanelDelegate")
        objc.protocolNamed("NSOutlineViewDataSource")
        objc.protocolNamed("NSOutlineViewDelegate")
        objc.protocolNamed("NSPasteboardItemDataProvider")
        objc.protocolNamed("NSPasteboardReading")
        objc.protocolNamed("NSPasteboardWriting")
        objc.protocolNamed("NSPathControlDelegate")
        objc.protocolNamed("NSPrintPanelAccessorizing")
        objc.protocolNamed("NSRuleEditorDelegate")
        objc.protocolNamed("NSTableViewDataSource")
        objc.protocolNamed("NSTableViewDelegate")
        objc.protocolNamed("NSTextAttachmentCell")
        objc.protocolNamed("NSTextDelegate")
        objc.protocolNamed("NSTextFieldDelegate")
        objc.protocolNamed("NSTextInput")
        objc.protocolNamed("NSTextInputClient")
        objc.protocolNamed("NSTextViewDelegate")
        objc.protocolNamed("NSToolbarDelegate")
        objc.protocolNamed("NSUserInterfaceValidations")
        objc.protocolNamed("NSValidatedUserInterfaceItem")
        objc.protocolNamed("NSWindowDelegate")

    @min_os_level("10.7")
    def testProtocols10_7(self):
        objc.protocolNamed("NSDraggingDestination")
        objc.protocolNamed("NSDraggingSource")
        objc.protocolNamed("NSPopoverDelegate")
        objc.protocolNamed("NSTextFinderBarContainer")
        objc.protocolNamed("NSTextLayoutOrientationProvider")
        objc.protocolNamed("NSUserInterfaceItemIdentification")
        objc.protocolNamed("NSWindowRestoration")

    @min_sdk_level("10.8")
    def testProtocols10_8(self):
        objc.protocolNamed("NSSharingServiceDelegate")
        objc.protocolNamed("NSSharingServicePickerDelegate")



import os

import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSHFSFileTypes(TestCase):
    def testFunctions(self):
        v = Foundation.NSHFSTypeCodeFromFileType("'rtfd'")
        self.assertIsInstance(v, int)
        w = Foundation.NSFileTypeForHFSTypeCode(v)
        self.assertIsInstance(w, str)
        self.assertEqual(w, "'rtfd'")

        fname = "/Library/Documentation/Acknowledgements.rtf"
        if not os.path.exists(fname):
            fname = "/Library/Documentation/AirPort Acknowledgements.rtf"
        if not os.path.exists(fname):
            fname = "/Library/Documentation//iPod/Acknowledgements.rtf"

        if not os.path.exists(fname):
            self.fail("test file doesn't exist")

        v = Foundation.NSHFSTypeOfFile(fname)
        self.assertIsInstance(v, str)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc


class TestNSPathCell(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSPathStyleStandard, 0)
        self.assertEqual(AppKit.NSPathStyleNavigationBar, 1)
        self.assertEqual(AppKit.NSPathStylePopUp, 2)

    def testMethods(self):
        m = AppKit.NSPathCell.setDoubleAction_.__metadata__()
        self.assertEqual(m["arguments"][2]["sel_of_type"], b"v@:@")

    @min_sdk_level("10.10")
    def testProtocols(self):
        objc.protocolNamed("NSPathCellDelegate")



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestXMLNode(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSXMLInvalidKind, 0)
        self.assertEqual(Foundation.NSXMLDocumentKind, 1)
        self.assertEqual(Foundation.NSXMLElementKind, 2)
        self.assertEqual(Foundation.NSXMLAttributeKind, 3)
        self.assertEqual(Foundation.NSXMLNamespaceKind, 4)
        self.assertEqual(Foundation.NSXMLProcessingInstructionKind, 5)
        self.assertEqual(Foundation.NSXMLCommentKind, 6)
        self.assertEqual(Foundation.NSXMLTextKind, 7)
        self.assertEqual(Foundation.NSXMLDTDKind, 8)
        self.assertEqual(Foundation.NSXMLEntityDeclarationKind, 9)
        self.assertEqual(Foundation.NSXMLAttributeDeclarationKind, 10)
        self.assertEqual(Foundation.NSXMLElementDeclarationKind, 11)
        self.assertEqual(Foundation.NSXMLNotationDeclarationKind, 12)

    def testOutputArgs(self):
        self.assertArgIsOut(Foundation.NSXMLNode.nodesForXPath_error_, 1)
        self.assertArgIsOut(Foundation.NSXMLNode.objectsForXQuery_constants_error_, 2)
        self.assertArgIsOut(Foundation.NSXMLNode.objectsForXQuery_error_, 1)
        self.assertArgIsBOOL(Foundation.NSXMLNode.setStringValue_resolvingEntities_, 1)
        self.assertArgIsBOOL(
            Foundation.NSXMLNode.canonicalXMLStringPreservingComments_, 0
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSVisualEffectView(TestCase):
    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertEqual(AppKit.NSVisualEffectMaterialAppearanceBased, 0)
        self.assertEqual(AppKit.NSVisualEffectMaterialLight, 1)
        self.assertEqual(AppKit.NSVisualEffectMaterialDark, 2)
        self.assertEqual(AppKit.NSVisualEffectMaterialTitlebar, 3)
        self.assertEqual(AppKit.NSVisualEffectMaterialSelection, 4)
        self.assertEqual(AppKit.NSVisualEffectMaterialMenu, 5)
        self.assertEqual(AppKit.NSVisualEffectMaterialPopover, 6)
        self.assertEqual(AppKit.NSVisualEffectMaterialSidebar, 7)
        self.assertEqual(AppKit.NSVisualEffectMaterialMediumLight, 8)
        self.assertEqual(AppKit.NSVisualEffectMaterialUltraDark, 9)
        self.assertEqual(AppKit.NSVisualEffectMaterialHeaderView, 10)
        self.assertEqual(AppKit.NSVisualEffectMaterialSheet, 11)
        self.assertEqual(AppKit.NSVisualEffectMaterialWindowBackground, 12)
        self.assertEqual(AppKit.NSVisualEffectMaterialHUDWindow, 13)
        self.assertEqual(AppKit.NSVisualEffectMaterialFullScreenUI, 15)
        self.assertEqual(AppKit.NSVisualEffectMaterialToolTip, 17)
        self.assertEqual(AppKit.NSVisualEffectMaterialContentBackground, 18)
        self.assertEqual(AppKit.NSVisualEffectMaterialUnderWindowBackground, 21)
        self.assertEqual(AppKit.NSVisualEffectMaterialUnderPageBackground, 22)

        self.assertEqual(AppKit.NSVisualEffectBlendingModeBehindWindow, 0)
        self.assertEqual(AppKit.NSVisualEffectBlendingModeWithinWindow, 1)

        self.assertEqual(AppKit.NSVisualEffectStateFollowsWindowActiveState, 0)
        self.assertEqual(AppKit.NSVisualEffectStateActive, 1)
        self.assertEqual(AppKit.NSVisualEffectStateInactive, 2)

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(AppKit.NSVisualEffectView.isEmphasized)
        self.assertArgIsBOOL(AppKit.NSVisualEffectView.setEmphasized_, 0)



#
#  AppController.py
#  ManualBindings
#
import objc
from Cocoa import (
    NSDecimal,
    NSError,
    NSLocalizedDescriptionKey,
    NSObject,
    NSValidatesImmediatelyBindingOption,
)


class AppController(NSObject):
    arrayController = objc.IBOutlet()
    selectedNameField = objc.IBOutlet()
    selectedPriceField = objc.IBOutlet()
    tableView = objc.IBOutlet()
    totalCountField = objc.IBOutlet()

    def awakeFromNib(self):
        self.totalCountField.bind_toObject_withKeyPath_options_(
            "value", self.arrayController, "arrangedObjects.@sum.price", None
        )
        bindingOptions = {}
        bindingOptions["NSNullPlaceholder"] = "No Name"
        self.selectedNameField.bind_toObject_withKeyPath_options_(
            "value", self.arrayController, "selection.name", bindingOptions
        )
        # binding for "name" column
        tableColumn = self.tableView.tableColumnWithIdentifier_("name")
        tableColumn.bind_toObject_withKeyPath_options_(
            "value", self.arrayController, "arrangedObjects.name", bindingOptions
        )

        # binding options for "price"
        del bindingOptions["NSNullPlaceholder"]
        bindingOptions[NSValidatesImmediatelyBindingOption] = True

        # binding for selected "price" field
        self.selectedPriceField.bind_toObject_withKeyPath_options_(
            "value", self.arrayController, "selection.price", bindingOptions
        )

        # binding for "price" column
        tableColumn = self.tableView.tableColumnWithIdentifier_("price")
        tableColumn.bind_toObject_withKeyPath_options_(
            "value", self.arrayController, "arrangedObjects.price", bindingOptions
        )

        # bind array controller to self's itemsArray
        # we use _k_itemsArray because Python does not have a separate
        # namespace for instance variables, and we are using accessors.
        self._k_itemsArray = []
        self.arrayController.bind_toObject_withKeyPath_options_(
            "contentArray", self, "self.itemsArray", None
        )

    @objc.accessor
    def countOfItemsArray(self):
        return len(self._k_itemsArray)

    @objc.accessor
    def objectInItemsArrayAtIndex_(self, index):
        return self._k_itemsArray[index]

    @objc.accessor
    def insertObject_inItemsArrayAtIndex_(self, obj, idx):
        self._k_itemsArray.insert(idx, obj)

    @objc.accessor
    def removeObjectFromItemsArrayAtIndex_(self, idx):
        del self._k_itemsArray[idx]

    @objc.accessor
    def replaceObjectInItemsArrayAtIndex_withObject_(self, idx, obj):
        self._k_itemsArray[idx] = obj


ITEM_ERROR_DOMAIN = "ITEM_ERROR_DOMAIN"
ITEM_NEGATIVE_PRICE = 10001


class Item(NSObject):
    def price(self):
        return getattr(self, "_k_price", NSDecimal("0.00"))

    def setPrice_(self, aPrice):
        self._k_price = aPrice

    def name(self):
        return getattr(self, "_k_name", None)

    def setName_(self, aName):
        self._k_name = aName

    @objc.accessor
    def validatePrice_error_(self, value, error):
        if value >= 0:
            return True, value, None

        errorString = "Price cannot be negative"
        userInfoDict = {NSLocalizedDescriptionKey: errorString}
        error = NSError.alloc().initWithDomain_code_userInfo_(
            ITEM_ERROR_DOMAIN, ITEM_NEGATIVE_PRICE, userInfoDict
        )

        return False, value, error



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSBrowserCell(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSBrowserCell.isLeaf)
        self.assertArgIsBOOL(AppKit.NSBrowserCell.setLeaf_, 0)
        self.assertResultIsBOOL(AppKit.NSBrowserCell.isLoaded)
        self.assertArgIsBOOL(AppKit.NSBrowserCell.setLoaded_, 0)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSSpeechSynthesizerHelper(AppKit.NSObject):
    def speechSynthesizer_didFinishSpeaking_(self, ss, b):
        pass

    def speechSynthesizer_willSpeakWord_ofString_(self, ss, w, s):
        pass

    def speechSynthesizer_willSpeakPhoneme_(self, ss, i):
        pass

    def speechSynthesizer_didEncounterErrorAtIndex_ofString_message_(self, ss, i, s, m):
        pass


class TestNSSpeechSynthesizer(TestCase):
    def testConstants(self):
        self.assertIsInstance(AppKit.NSVoiceName, str)
        self.assertIsInstance(AppKit.NSVoiceIdentifier, str)
        self.assertIsInstance(AppKit.NSVoiceAge, str)
        self.assertIsInstance(AppKit.NSVoiceGender, str)
        self.assertIsInstance(AppKit.NSVoiceDemoText, str)

        self.assertIsInstance(AppKit.NSVoiceGenderNeuter, str)
        self.assertIsInstance(AppKit.NSVoiceGenderMale, str)
        self.assertIsInstance(AppKit.NSVoiceGenderFemale, str)
        self.assertIsInstance(AppKit.NSVoiceLanguage, str)

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertEqual(AppKit.NSSpeechImmediateBoundary, 0)
        self.assertEqual(AppKit.NSSpeechWordBoundary, 1)
        self.assertEqual(AppKit.NSSpeechSentenceBoundary, 2)

        self.assertIsInstance(AppKit.NSVoiceLocaleIdentifier, str)
        self.assertIsInstance(AppKit.NSVoiceSupportedCharacters, str)
        self.assertIsInstance(AppKit.NSVoiceIndividuallySpokenCharacters, str)

        self.assertIsInstance(AppKit.NSSpeechStatusProperty, str)
        self.assertIsInstance(AppKit.NSSpeechErrorsProperty, str)
        self.assertIsInstance(AppKit.NSSpeechInputModeProperty, str)
        self.assertIsInstance(AppKit.NSSpeechCharacterModeProperty, str)
        self.assertIsInstance(AppKit.NSSpeechNumberModeProperty, str)
        self.assertIsInstance(AppKit.NSSpeechRateProperty, str)
        self.assertIsInstance(AppKit.NSSpeechPitchBaseProperty, str)
        self.assertIsInstance(AppKit.NSSpeechPitchModProperty, str)
        self.assertIsInstance(AppKit.NSSpeechVolumeProperty, str)
        self.assertIsInstance(AppKit.NSSpeechSynthesizerInfoProperty, str)
        self.assertIsInstance(AppKit.NSSpeechRecentSyncProperty, str)
        self.assertIsInstance(AppKit.NSSpeechPhonemeSymbolsProperty, str)
        self.assertIsInstance(AppKit.NSSpeechCurrentVoiceProperty, str)
        self.assertIsInstance(AppKit.NSSpeechCommandDelimiterProperty, str)
        self.assertIsInstance(AppKit.NSSpeechResetProperty, str)
        self.assertIsInstance(AppKit.NSSpeechOutputToFileURLProperty, str)

        self.assertIsInstance(AppKit.NSSpeechModeText, str)
        self.assertIsInstance(AppKit.NSSpeechModePhoneme, str)

        self.assertIsInstance(AppKit.NSSpeechModeNormal, str)
        self.assertIsInstance(AppKit.NSSpeechModeLiteral, str)

        self.assertIsInstance(AppKit.NSSpeechStatusOutputBusy, str)
        self.assertIsInstance(AppKit.NSSpeechStatusOutputPaused, str)
        self.assertIsInstance(AppKit.NSSpeechStatusNumberOfCharactersLeft, str)
        self.assertIsInstance(AppKit.NSSpeechStatusPhonemeCode, str)

        self.assertIsInstance(AppKit.NSSpeechErrorCount, str)
        self.assertIsInstance(AppKit.NSSpeechErrorOldestCode, str)
        self.assertIsInstance(AppKit.NSSpeechErrorOldestCharacterOffset, str)
        self.assertIsInstance(AppKit.NSSpeechErrorNewestCode, str)
        self.assertIsInstance(AppKit.NSSpeechErrorNewestCharacterOffset, str)

        self.assertIsInstance(AppKit.NSSpeechSynthesizerInfoIdentifier, str)
        self.assertIsInstance(AppKit.NSSpeechSynthesizerInfoVersion, str)

        self.assertIsInstance(AppKit.NSSpeechPhonemeInfoOpcode, str)
        self.assertIsInstance(AppKit.NSSpeechPhonemeInfoSymbol, str)
        self.assertIsInstance(AppKit.NSSpeechPhonemeInfoExample, str)
        self.assertIsInstance(AppKit.NSSpeechPhonemeInfoHiliteStart, str)
        self.assertIsInstance(AppKit.NSSpeechPhonemeInfoHiliteEnd, str)

        self.assertIsInstance(AppKit.NSSpeechCommandPrefix, str)
        self.assertIsInstance(AppKit.NSSpeechCommandSuffix, str)

        self.assertIsInstance(AppKit.NSSpeechDictionaryLocaleIdentifier, str)
        self.assertIsInstance(AppKit.NSSpeechDictionaryModificationDate, str)
        self.assertIsInstance(AppKit.NSSpeechDictionaryPronunciations, str)
        self.assertIsInstance(AppKit.NSSpeechDictionaryAbbreviations, str)
        self.assertIsInstance(AppKit.NSSpeechDictionaryEntrySpelling, str)
        self.assertIsInstance(AppKit.NSSpeechDictionaryEntryPhonemes, str)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSSpeechSynthesizer.startSpeakingString_)
        self.assertResultIsBOOL(AppKit.NSSpeechSynthesizer.startSpeakingString_toURL_)
        self.assertResultIsBOOL(AppKit.NSSpeechSynthesizer.isSpeaking)
        self.assertResultIsBOOL(AppKit.NSSpeechSynthesizer.setVoice_)
        self.assertResultIsBOOL(AppKit.NSSpeechSynthesizer.usesFeedbackWindow)
        self.assertArgIsBOOL(AppKit.NSSpeechSynthesizer.setUsesFeedbackWindow_, 0)
        self.assertResultIsBOOL(AppKit.NSSpeechSynthesizer.setObject_forProperty_error_)
        self.assertArgIsOut(AppKit.NSSpeechSynthesizer.setObject_forProperty_error_, 2)
        self.assertArgIsOut(AppKit.NSSpeechSynthesizer.objectForProperty_error_, 1)
        self.assertResultIsBOOL(AppKit.NSSpeechSynthesizer.isAnyApplicationSpeaking)

    def testProtocol(self):
        self.assertArgIsBOOL(
            TestNSSpeechSynthesizerHelper.speechSynthesizer_didFinishSpeaking_, 1
        )
        self.assertArgHasType(
            TestNSSpeechSynthesizerHelper.speechSynthesizer_willSpeakWord_ofString_,
            1,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSSpeechSynthesizerHelper.speechSynthesizer_willSpeakPhoneme_,
            1,
            objc._C_SHT,
        )

    @min_os_level("10.5")
    def testProtocol10_5(self):
        self.assertArgHasType(
            TestNSSpeechSynthesizerHelper.speechSynthesizer_didEncounterErrorAtIndex_ofString_message_,  # noqa: B950
            1,
            objc._C_NSUInteger,
        )

    @min_sdk_level("10.9")
    def testProtocol10_9(self):
        objc.protocolNamed("NSSpeechSynthesizerDelegate")



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSForm(TestCase):
    def testMethods(self):
        self.assertArgIsBOOL(AppKit.NSForm.setBordered_, 0)
        self.assertArgIsBOOL(AppKit.NSForm.setBezeled_, 0)



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestEAManager(TestCase):
    def testContants(self):
        self.assertIsInstance(
            Foundation.NSAppleEventManagerWillProcessFirstEventNotification, str
        )
        self.assertIsInstance(Foundation.NSAppleEventTimeOutDefault, float)
        self.assertIsInstance(Foundation.NSAppleEventTimeOutNone, float)

    def testOpaque(self):
        self.assertHasAttr(Foundation, "NSAppleEventManagerSuspensionID")
        self.assertHasAttr(Foundation.NSAppleEventManagerSuspensionID, "__pointer__")

    def testMethods(self):
        self.assertArgIsSEL(
            Foundation.NSAppleEventManager.setEventHandler_andSelector_forEventClass_andEventID_,  # noqa: B950
            1,
            b"v@:@@",
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_sdk_level, min_os_level
import objc


class TestNSTextAttachmentHelper(AppKit.NSObject):
    def drawWithFrame_inView_(self, fr, vi):
        pass

    def wantsToTrackMouse(self):
        return 1

    def highlight_withFrame_inView_(self, fl, fr, vi):
        pass

    def trackMouse_inRect_ofView_untilMouseUp_(self, ev, fr, vi, fl):
        pass

    def cellSize(self):
        return 1

    def cellBaselineOffset(self):
        return 1

    def drawWithFrame_inView_characterIndex_(self, fr, vi, i):
        pass

    def drawWithFrame_inView_characterIndex_layoutManager_(self, fr, vi, i, lm):
        pass

    def wantsToTrackMouseForEvent_inRect_ofView_atCharacterIndex_(self, ev, fr, vi, i):
        return 1

    def trackMouse_inRect_ofView_atCharacterIndex_untilMouseUp_(
        self, ev, fr, vi, i, fl
    ):
        return 1

    def cellFrameForTextContainer_proposedLineFragment_glyphPosition_characterIndex_(
        self, tc, fr, po, i
    ):
        return 1

    def imageForBounds_textContainer_characterIndex_(self, b, c, i):
        return 1

    def attachmentBoundsForTextContainer_proposedLineFragment_glyphPosition_characterIndex_(
        self, c, f, p, i
    ):
        return 1


class TestNSTextAttachment(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSAttachmentCharacter, chr(0xFFFC))

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSTextAttachmentCell.wantsToTrackMouse)
        self.assertArgIsBOOL(AppKit.NSTextAttachmentCell.highlight_withFrame_inView_, 0)
        self.assertResultIsBOOL(
            AppKit.NSTextAttachmentCell.trackMouse_inRect_ofView_untilMouseUp_
        )
        self.assertArgIsBOOL(
            AppKit.NSTextAttachmentCell.trackMouse_inRect_ofView_untilMouseUp_, 3
        )
        self.assertResultIsBOOL(
            AppKit.NSTextAttachmentCell.wantsToTrackMouseForEvent_inRect_ofView_atCharacterIndex_  # noqa: B950
        )
        self.assertResultIsBOOL(
            AppKit.NSTextAttachmentCell.trackMouse_inRect_ofView_atCharacterIndex_untilMouseUp_
        )
        self.assertArgIsBOOL(
            AppKit.NSTextAttachmentCell.trackMouse_inRect_ofView_atCharacterIndex_untilMouseUp_,
            4,
        )

    @min_os_level("12.0")
    def testMethods12_0(self):
        self.assertResultIsBOOL(AppKit.NSTextAttachment.allowsTextAttachmentView)
        self.assertArgIsBOOL(AppKit.NSTextAttachment.setAllowsTextAttachmentView_, 0)
        self.assertResultIsBOOL(AppKit.NSTextAttachment.usesTextAttachmentView)

        self.assertResultIsBOOL(
            AppKit.NSTextAttachmentViewProvider.tracksTextAttachmentViewBounds
        )
        self.assertArgIsBOOL(
            AppKit.NSTextAttachmentViewProvider.setTracksTextAttachmentViewBounds_, 0
        )

    @min_sdk_level("10.11")
    def testProtocolObjects(self):
        objc.protocolNamed("NSTextAttachmentContainer")
        objc.protocolNamed("NSTextAttachmentCell")

    def testProtocols(self):
        self.assertArgHasType(
            TestNSTextAttachmentHelper.imageForBounds_textContainer_characterIndex_,
            0,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextAttachmentHelper.imageForBounds_textContainer_characterIndex_,
            2,
            objc._C_NSUInteger,
        )

        self.assertArgHasType(
            TestNSTextAttachmentHelper.attachmentBoundsForTextContainer_proposedLineFragment_glyphPosition_characterIndex_,  # noqa: B950
            1,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextAttachmentHelper.attachmentBoundsForTextContainer_proposedLineFragment_glyphPosition_characterIndex_,  # noqa: B950
            2,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextAttachmentHelper.attachmentBoundsForTextContainer_proposedLineFragment_glyphPosition_characterIndex_,  # noqa: B950
            3,
            objc._C_NSUInteger,
        )

        self.assertArgHasType(
            TestNSTextAttachmentHelper.drawWithFrame_inView_,
            0,
            AppKit.NSRect.__typestr__,
        )
        self.assertResultIsBOOL(TestNSTextAttachmentHelper.wantsToTrackMouse)
        self.assertArgIsBOOL(TestNSTextAttachmentHelper.highlight_withFrame_inView_, 0)
        self.assertArgHasType(
            TestNSTextAttachmentHelper.highlight_withFrame_inView_,
            1,
            AppKit.NSRect.__typestr__,
        )
        self.assertResultIsBOOL(
            TestNSTextAttachmentHelper.trackMouse_inRect_ofView_untilMouseUp_
        )
        self.assertArgHasType(
            TestNSTextAttachmentHelper.trackMouse_inRect_ofView_untilMouseUp_,
            1,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgIsBOOL(
            TestNSTextAttachmentHelper.trackMouse_inRect_ofView_untilMouseUp_, 3
        )
        self.assertResultHasType(
            TestNSTextAttachmentHelper.cellSize, AppKit.NSSize.__typestr__
        )
        self.assertResultHasType(
            TestNSTextAttachmentHelper.cellBaselineOffset, AppKit.NSPoint.__typestr__
        )
        self.assertArgHasType(
            TestNSTextAttachmentHelper.drawWithFrame_inView_characterIndex_,
            0,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextAttachmentHelper.drawWithFrame_inView_characterIndex_,
            2,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSTextAttachmentHelper.drawWithFrame_inView_characterIndex_layoutManager_,
            0,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextAttachmentHelper.drawWithFrame_inView_characterIndex_layoutManager_,
            2,
            objc._C_NSUInteger,
        )
        self.assertResultIsBOOL(
            TestNSTextAttachmentHelper.wantsToTrackMouseForEvent_inRect_ofView_atCharacterIndex_
        )
        self.assertArgHasType(
            TestNSTextAttachmentHelper.wantsToTrackMouseForEvent_inRect_ofView_atCharacterIndex_,  # noqa: B950
            1,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextAttachmentHelper.wantsToTrackMouseForEvent_inRect_ofView_atCharacterIndex_,  # noqa: B950
            3,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSTextAttachmentHelper.trackMouse_inRect_ofView_atCharacterIndex_untilMouseUp_,
            1,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextAttachmentHelper.trackMouse_inRect_ofView_atCharacterIndex_untilMouseUp_,
            3,
            objc._C_NSUInteger,
        )

        self.assertResultHasType(
            TestNSTextAttachmentHelper.cellFrameForTextContainer_proposedLineFragment_glyphPosition_characterIndex_,  # noqa: B950
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextAttachmentHelper.cellFrameForTextContainer_proposedLineFragment_glyphPosition_characterIndex_,  # noqa: B950
            1,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextAttachmentHelper.cellFrameForTextContainer_proposedLineFragment_glyphPosition_characterIndex_,  # noqa: B950
            2,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestNSTextAttachmentHelper.cellFrameForTextContainer_proposedLineFragment_glyphPosition_characterIndex_,  # noqa: B950
            3,
            objc._C_NSUInteger,
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSTabViewController(TestCase):
    @min_os_level("10.10")
    def testConstants(self):
        self.assertEqual(AppKit.NSTabViewControllerTabStyleSegmentedControlOnTop, 0)
        self.assertEqual(AppKit.NSTabViewControllerTabStyleSegmentedControlOnBottom, 1)
        self.assertEqual(AppKit.NSTabViewControllerTabStyleToolbar, 2)
        self.assertEqual(AppKit.NSTabViewControllerTabStyleUnspecified, -1)

    @min_os_level("10.10")
    def testMethods(self):
        self.assertResultIsBOOL(
            AppKit.NSTabViewController.canPropagateSelectedChildViewControllerTitle
        )
        self.assertArgIsBOOL(
            AppKit.NSTabViewController.setCanPropagateSelectedChildViewControllerTitle_,
            0,
        )

        self.assertResultIsBOOL(
            AppKit.NSTabViewController.tabView_shouldSelectTabViewItem_
        )

        self.assertArgIsBOOL(
            AppKit.NSTabViewController.toolbar_itemForItemIdentifier_willBeInsertedIntoToolbar_,
            2,
        )



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSUserDefaultsController(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSUserDefaultsController.appliesImmediately)
        self.assertArgIsBOOL(AppKit.NSUserDefaultsController.setAppliesImmediately_, 0)
        self.assertResultIsBOOL(AppKit.NSUserDefaultsController.hasUnappliedChanges)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TesNSScrollView(TestCase):
    def testMethods(self):
        self.assertArgIsBOOL(
            AppKit.NSScrollView.frameSizeForContentSize_hasHorizontalScroller_hasVerticalScroller_borderType_,  # noqa: B950
            1,
        )
        self.assertArgIsBOOL(
            AppKit.NSScrollView.frameSizeForContentSize_hasHorizontalScroller_hasVerticalScroller_borderType_,  # noqa: B950
            2,
        )
        self.assertArgIsBOOL(
            AppKit.NSScrollView.contentSizeForFrameSize_hasHorizontalScroller_hasVerticalScroller_borderType_,  # noqa: B950
            1,
        )
        self.assertArgIsBOOL(
            AppKit.NSScrollView.contentSizeForFrameSize_hasHorizontalScroller_hasVerticalScroller_borderType_,  # noqa: B950
            2,
        )

        self.assertResultIsBOOL(AppKit.NSScrollView.drawsBackground)
        self.assertArgIsBOOL(AppKit.NSScrollView.setDrawsBackground_, 0)
        self.assertResultIsBOOL(AppKit.NSScrollView.hasVerticalScroller)
        self.assertArgIsBOOL(AppKit.NSScrollView.setHasVerticalScroller_, 0)
        self.assertResultIsBOOL(AppKit.NSScrollView.hasHorizontalScroller)
        self.assertArgIsBOOL(AppKit.NSScrollView.setHasHorizontalScroller_, 0)
        self.assertResultIsBOOL(AppKit.NSScrollView.autohidesScrollers)
        self.assertArgIsBOOL(AppKit.NSScrollView.setAutohidesScrollers_, 0)
        self.assertResultIsBOOL(AppKit.NSScrollView.scrollsDynamically)
        self.assertArgIsBOOL(AppKit.NSScrollView.setScrollsDynamically_, 0)
        self.assertResultIsBOOL(AppKit.NSScrollView.rulersVisible)
        self.assertArgIsBOOL(AppKit.NSScrollView.setRulersVisible_, 0)
        self.assertResultIsBOOL(AppKit.NSScrollView.hasHorizontalRuler)
        self.assertArgIsBOOL(AppKit.NSScrollView.setHasHorizontalRuler_, 0)
        self.assertResultIsBOOL(AppKit.NSScrollView.hasVerticalRuler)
        self.assertArgIsBOOL(AppKit.NSScrollView.setHasVerticalRuler_, 0)

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertArgIsBOOL(
            AppKit.NSScrollView.setAutomaticallyAdjustsContentInsets_, 0
        )
        self.assertResultIsBOOL(AppKit.NSScrollView.automaticallyAdjustsContentInsets)

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertIsInstance(AppKit.NSScrollViewWillStartLiveScrollNotification, str)
        self.assertIsInstance(AppKit.NSScrollViewDidLiveScrollNotification, str)
        self.assertIsInstance(AppKit.NSScrollViewDidEndLiveScrollNotification, str)

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertArgIsBOOL(AppKit.NSScrollView.setAllowsMagnification_, 0)
        self.assertResultIsBOOL(AppKit.NSScrollView.allowsMagnification)

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertIsInstance(AppKit.NSScrollViewWillStartLiveMagnifyNotification, str)
        self.assertIsInstance(AppKit.NSScrollViewDidEndLiveMagnifyNotification, str)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(AppKit.NSScrollElasticityAutomatic, 0)
        self.assertEqual(AppKit.NSScrollElasticityNone, 1)
        self.assertEqual(AppKit.NSScrollElasticityAllowed, 2)

        self.assertEqual(AppKit.NSScrollViewFindBarPositionAboveHorizontalRuler, 0)
        self.assertEqual(AppKit.NSScrollViewFindBarPositionAboveContent, 1)
        self.assertEqual(AppKit.NSScrollViewFindBarPositionBelowContent, 2)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(AppKit.NSScrollView.usesPredominantAxisScrolling)
        self.assertArgIsBOOL(AppKit.NSScrollView.setUsesPredominantAxisScrolling_, 0)



#
#  FilteringArrayController.py
#  FilteringController
#
#  Converted by u.fiedler on 05.02.05.
#
#  The original version was written in Objective-C by Malcolm Crawford
#  at http://homepage.mac.com/mmalc/CocoaExamples/controllers.html

from Cocoa import NSArrayController
from objc import super


class FilteringArrayController(NSArrayController):
    _k_searchString = ""

    def search_(self, sender):
        self.setSearchString_(sender.stringValue())
        self.rearrangeObjects()

    def newObject(self):
        """
        Creates and returns a new object of the class specified by objectClass.
        Set default values, and keep reference to new object -- see arrangeObjects_
        """
        self.newObj = super().newObject()
        self.newObj.setValue_forKey_("First", "firstName")
        self.newObj.setValue_forKey_("Last", "lastName")
        return self.newObj

    def arrangeObjects_(self, objects):
        if self._k_searchString is None or self._k_searchString == "":
            self.newObj = None
            return super().arrangeObjects_(objects)

        # Create array of objects that match search string.
        # Also add any newly-created object unconditionally:
        # (a) You'll get an error if a newly-added object isn't added to
        # arrangedObjects.
        # (b) The user will see newly-added objects even if they don't
        # match the search term.

        matchedObjects = []
        lowerSearch = self._k_searchString.lower()
        for item in objects:
            if item == self.newObj:
                # if the item has just been created, add it unconditionally
                matchedObjects.append(item)
                self.newObj = None
            else:
                lowerName = item.valueForKeyPath_("firstName").lower()
                if lowerSearch in lowerName:
                    matchedObjects.append(item)
                else:
                    lowerName = item.valueForKeyPath_("lastName").lower()
                    if lowerSearch in lowerName:
                        matchedObjects.append(item)
        return super().arrangeObjects_(matchedObjects)

    def searchString(self):
        return self._k_searchString

    def setSearchString_(self, newStr):
        self._k_searchString = newStr



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSPersonNameComponentsFormatter(TestCase):
    @min_os_level("10.11")
    def testConstants(self):
        self.assertEqual(Foundation.NSPersonNameComponentsFormatterStyleDefault, 0)
        self.assertEqual(Foundation.NSPersonNameComponentsFormatterStyleShort, 1)
        self.assertEqual(Foundation.NSPersonNameComponentsFormatterStyleMedium, 2)
        self.assertEqual(Foundation.NSPersonNameComponentsFormatterStyleLong, 3)
        self.assertEqual(Foundation.NSPersonNameComponentsFormatterStyleAbbreviated, 4)

        self.assertEqual(Foundation.NSPersonNameComponentsFormatterPhonetic, 1 << 1)

        self.assertIsInstance(Foundation.NSPersonNameComponentKey, str)
        self.assertIsInstance(Foundation.NSPersonNameComponentGivenName, str)
        self.assertIsInstance(Foundation.NSPersonNameComponentFamilyName, str)
        self.assertIsInstance(Foundation.NSPersonNameComponentMiddleName, str)
        self.assertIsInstance(Foundation.NSPersonNameComponentPrefix, str)
        self.assertIsInstance(Foundation.NSPersonNameComponentSuffix, str)
        self.assertIsInstance(Foundation.NSPersonNameComponentNickname, str)
        self.assertIsInstance(Foundation.NSPersonNameComponentDelimiter, str)

    @min_os_level("10.11")
    def testOutput(self):
        self.assertResultIsBOOL(
            Foundation.NSPersonNameComponentsFormatter.getObjectValue_forString_errorDescription_  # noqa: B950
        )
        self.assertArgIsOut(
            Foundation.NSPersonNameComponentsFormatter.getObjectValue_forString_errorDescription_,  # noqa: B950
            0,
        )
        self.assertArgIsOut(
            Foundation.NSPersonNameComponentsFormatter.getObjectValue_forString_errorDescription_,  # noqa: B950
            2,
        )

        self.assertResultIsBOOL(Foundation.NSPersonNameComponentsFormatter.isPhonetic)
        self.assertArgIsBOOL(Foundation.NSPersonNameComponentsFormatter.setPhonetic_, 0)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSSliderAccessory(TestCase):
    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(AppKit.NSSliderAccessory.isEnabled)
        self.assertArgIsBOOL(AppKit.NSSliderAccessory.setEnabled_, 0)

        self.assertArgIsBlock(
            AppKit.NSSliderAccessoryBehavior.behaviorWithHandler_, 0, b"v@"
        )



import CoreFoundation
from PyObjCTools.TestSupport import TestCase, expectedFailure


class TestXMLNode(TestCase):
    # NOTE: This doesn't actually test the API

    def testTypes(self):
        self.assertIsCFType(CoreFoundation.CFXMLNodeRef)

    def testConstants(self):
        self.assertEqual(CoreFoundation.kCFXMLNodeCurrentVersion, 1)

        self.assertEqual(CoreFoundation.kCFXMLNodeTypeDocument, 1)
        self.assertEqual(CoreFoundation.kCFXMLNodeTypeElement, 2)
        self.assertEqual(CoreFoundation.kCFXMLNodeTypeAttribute, 3)
        self.assertEqual(CoreFoundation.kCFXMLNodeTypeProcessingInstruction, 4)
        self.assertEqual(CoreFoundation.kCFXMLNodeTypeComment, 5)
        self.assertEqual(CoreFoundation.kCFXMLNodeTypeText, 6)
        self.assertEqual(CoreFoundation.kCFXMLNodeTypeCDATASection, 7)
        self.assertEqual(CoreFoundation.kCFXMLNodeTypeDocumentFragment, 8)
        self.assertEqual(CoreFoundation.kCFXMLNodeTypeEntity, 9)
        self.assertEqual(CoreFoundation.kCFXMLNodeTypeEntityReference, 10)
        self.assertEqual(CoreFoundation.kCFXMLNodeTypeDocumentType, 11)
        self.assertEqual(CoreFoundation.kCFXMLNodeTypeWhitespace, 12)
        self.assertEqual(CoreFoundation.kCFXMLNodeTypeNotation, 13)
        self.assertEqual(CoreFoundation.kCFXMLNodeTypeElementTypeDeclaration, 14)
        self.assertEqual(CoreFoundation.kCFXMLNodeTypeAttributeListDeclaration, 15)

        self.assertEqual(CoreFoundation.kCFXMLEntityTypeParameter, 0)
        self.assertEqual(CoreFoundation.kCFXMLEntityTypeParsedInternal, 1)
        self.assertEqual(CoreFoundation.kCFXMLEntityTypeParsedExternal, 2)
        self.assertEqual(CoreFoundation.kCFXMLEntityTypeUnparsed, 3)
        self.assertEqual(CoreFoundation.kCFXMLEntityTypeCharacter, 4)

    def testStructs(self):
        return

        o = CoreFoundation.CFXMLElementInfo()
        self.assertHasAttr(o, "attributes")
        self.assertHasAttr(o, "attributeOrder")
        self.assertHasAttr(o, "isEmpty")
        self.assertHasAttr(o, "_reserved")
        o = CoreFoundation.CFXMLProcessingInstructionInfo()
        self.assertHasAttr(o, "dataString")
        o = CoreFoundation.CFXMLDocumentInfo()
        self.assertHasAttr(o, "sourceURL")
        self.assertHasAttr(o, "encoding")
        o = CoreFoundation.CFXMLExternalID()
        self.assertHasAttr(o, "systemID")
        self.assertHasAttr(o, "publicID")
        o = CoreFoundation.CFXMLDocumentTypeInfo()
        self.assertHasAttr(o, "externalID")
        o = CoreFoundation.CFXMLNotationInfo()
        self.assertHasAttr(o, "externalID")
        o = CoreFoundation.CFXMLElementTypeDeclarationInfo()
        self.assertHasAttr(o, "contentDescription")
        o = CoreFoundation.CFXMLAttributeDeclarationInfo()
        self.assertHasAttr(o, "attributeName")
        self.assertHasAttr(o, "typeString")
        self.assertHasAttr(o, "defaultString")
        o = CoreFoundation.CFXMLAttributeListDeclarationInfo()
        self.assertHasAttr(o, "numberOfAttributes")
        self.assertHasAttr(o, "attributes")
        o = CoreFoundation.CFXMLEntityInfo()
        self.assertHasAttr(o, "entityType")
        self.assertHasAttr(o, "replacementText")
        self.assertHasAttr(o, "entityID")
        self.assertHasAttr(o, "notationName")
        o = CoreFoundation.CFXMLEntityReferenceInfo()
        self.assertHasAttr(o, "entityType")

    def testFunctions(self):
        self.assertIsInstance(CoreFoundation.CFXMLNodeGetTypeID(), int)

        # CoreFoundation.CFXMLNodeCreate: requires manual binding
        # CoreFoundation.CFXMLNodeGetInfoPtr: likewise
        # Add tests that create all valid types of nodes with there additional info and
        # try to extract the information.

        self.assertResultIsCFRetained(CoreFoundation.CFXMLNodeCreateCopy)
        self.assertResultIsCFRetained(CoreFoundation.CFXMLTreeCreateWithNode)
        CoreFoundation.CFXMLNodeGetTypeCode
        CoreFoundation.CFXMLNodeGetString
        CoreFoundation.CFXMLNodeGetVersion
        CoreFoundation.CFXMLTreeGetNode

    @expectedFailure
    def testMissingWrappers(self):
        self.fail("CFXML requires manual wrappers (low prio)")



"""
Script for building the example.

Usage:
    python3 setup.py py2app
"""
from setuptools import setup

plist = {
    "CFBundleDocumentTypes": [
        {
            "CFBundleTypeExtensions": ["txt", "text", "*"],
            "CFBundleTypeName": "Text File",
            "CFBundleTypeRole": "Editor",
            "NSDocumentClass": "TinyTinyDocument",
        }
    ]
}

setup(
    name="Tiny Tiny Edit",
    app=["TinyTinyEdit.py"],
    data_files=["MainMenu.nib", "TinyTinyDocument.nib"],
    options={"py2app": {"plist": plist}},
    setup_requires=["py2app", "pyobjc-framework-Cocoa"],
)



import sys

import AppKit

# import classes required to start application
import WSTApplicationDelegateClass  # noqa F401
import WSTConnectionWindowControllerClass  # noqa F401

# pass control to the AppKit
sys.exit(AppKit.NSApplicationMain(sys.argv))



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSProgress(TestCase):
    @min_os_level("10.9")
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSProgress.isCancellable)
        self.assertArgIsBOOL(AppKit.NSProgress.setCancellable_, 0)

        self.assertResultIsBOOL(AppKit.NSProgress.isPausable)
        self.assertArgIsBOOL(AppKit.NSProgress.setPausable_, 0)

        self.assertResultIsBOOL(AppKit.NSProgress.isCancelled)
        self.assertResultIsBOOL(AppKit.NSProgress.isPaused)

        self.assertResultIsBlock(AppKit.NSProgress.cancellationHandler, b"v")
        self.assertArgIsBlock(AppKit.NSProgress.setCancellationHandler_, 0, b"v")

        self.assertResultIsBlock(AppKit.NSProgress.pausingHandler, b"v")
        self.assertArgIsBlock(AppKit.NSProgress.setPausingHandler_, 0, b"v")

        self.assertResultIsBOOL(AppKit.NSProgress.isIndeterminate)

        self.assertArgIsBlock(
            AppKit.NSProgress.addSubscriberForFileURL_withPublishingHandler_, 1, b"@?@"
        )

        self.assertResultIsBOOL(AppKit.NSProgress.isOld)

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertResultIsBlock(AppKit.NSProgress.resumingHandler, b"v")
        self.assertArgIsBlock(AppKit.NSProgress.setResumingHandler_, 0, b"v")

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsBlock(
            AppKit.NSProgress.performAsCurrentWithPendingUnitCount_usingBlock_, 1, b"v"
        )

    @min_sdk_level("10.11")
    def testProtocolObjects(self):
        objc.protocolNamed("NSProgressReporting")

    @min_os_level("10.9")
    def testConstants(self):
        self.assertIsInstance(AppKit.NSProgressEstimatedTimeRemainingKey, str)
        self.assertIsInstance(AppKit.NSProgressThroughputKey, str)
        self.assertIsInstance(AppKit.NSProgressKindFile, str)
        self.assertIsInstance(AppKit.NSProgressFileOperationKindKey, str)
        self.assertIsInstance(AppKit.NSProgressFileOperationKindDownloading, str)
        self.assertIsInstance(
            AppKit.NSProgressFileOperationKindDecompressingAfterDownloading, str
        )
        self.assertIsInstance(AppKit.NSProgressFileOperationKindReceiving, str)
        self.assertIsInstance(AppKit.NSProgressFileOperationKindCopying, str)
        self.assertIsInstance(AppKit.NSProgressFileURLKey, str)
        self.assertIsInstance(AppKit.NSProgressFileTotalCountKey, str)
        self.assertIsInstance(AppKit.NSProgressFileCompletedCountKey, str)
        self.assertIsInstance(AppKit.NSProgressFileAnimationImageKey, str)
        self.assertIsInstance(AppKit.NSProgressFileAnimationImageOriginalRectKey, str)
        self.assertIsInstance(AppKit.NSProgressFileIconKey, str)

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(AppKit.NSProgressFileOperationKindUploading, str)

    @min_os_level("12.0")
    def testConstants12_0(self):
        self.assertIsInstance(AppKit.NSProgressFileOperationKindDuplicating, str)



"""
Script for building the example.

Usage:
    python3 setup.py py2app
"""
from setuptools import setup

plist = {
    "CFBundleIdentifier": "net.sf.pyobjc.PyObjCSimpleService",
    "CFBundleName": "PyObjCSimpleService",
    "LSBackgroundOnly": 1,
    "NSServices": [
        {
            "NSKeyEquivalent": {"default": "F"},
            "NSMenuItem": {"default": "Open File"},
            "NSMessage": "doOpenFileService",
            "NSPortName": "PyObjCSimpleService",
            "NSSendTypes": ["NSStringPboardType"],
        },
        {
            "NSMenuItem": {"default": "Capitalize String"},
            "NSMessage": "doCapitalizeService",
            "NSPortName": "PyObjCSimpleService",
            "NSReturnTypes": ["NSStringPboardType"],
            "NSSendTypes": ["NSStringPboardType"],
        },
    ],
}


setup(
    name="Simple Service",
    app=["SimpleService_main.py"],
    options={"py2app": {"plist": plist}},
    setup_requires=["py2app", "pyobjc-framework-Cocoa"],
)



import CoreFoundation
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure


class TestMessagePort(TestCase):
    def testTypes(self):
        self.assertIsCFType(CoreFoundation.CFMessagePortRef)

    def testConstants(self):
        self.assertEqual(CoreFoundation.kCFMessagePortSuccess, 0)
        self.assertEqual(CoreFoundation.kCFMessagePortSendTimeout, -1)
        self.assertEqual(CoreFoundation.kCFMessagePortReceiveTimeout, -2)
        self.assertEqual(CoreFoundation.kCFMessagePortIsInvalid, -3)
        self.assertEqual(CoreFoundation.kCFMessagePortTransportError, -4)
        self.assertEqual(CoreFoundation.kCFMessagePortBecameInvalidError, -5)

    @min_os_level("10.6")
    @expectedFailure
    def testFunctions10_6(self):
        self.fail("CFMessagePortSetDispatchQueue: dispatch_queue_t not wrapped yet")

    def testTypeID(self):
        self.assertIsInstance(CoreFoundation.CFMessagePortGetTypeID(), int)

    def testInteraction(self):
        class Context:
            pass

        context = Context()

        def callout(port, messageid, data, info):
            pass

        port, shouldFree = CoreFoundation.CFMessagePortCreateLocal(
            None, "name", callout, context, None
        )
        self.assertIsInstance(port, CoreFoundation.CFMessagePortRef)
        self.assertIs(shouldFree is True or shouldFree, False)
        self.assertFalse(CoreFoundation.CFMessagePortIsRemote(port))
        ctx = CoreFoundation.CFMessagePortGetContext(port, None)
        self.assertIs(ctx, context)

        port2 = CoreFoundation.CFMessagePortCreateRemote(None, "name")
        self.assertIsInstance(port2, CoreFoundation.CFMessagePortRef)
        self.assertResultIsBOOL(CoreFoundation.CFMessagePortIsRemote)
        self.assertTrue(CoreFoundation.CFMessagePortIsRemote(port2))
        self.assertTrue(CoreFoundation.CFMessagePortGetName(port2), "name")

        CoreFoundation.CFMessagePortSetName(port2, "newname")
        self.assertTrue(CoreFoundation.CFMessagePortGetName(port2), "newname")

        cb = CoreFoundation.CFMessagePortGetInvalidationCallBack(port)
        self.assertIs(cb, None)
        global didInvalidate
        didInvalidate = False

        @objc.callbackFor(CoreFoundation.CFMessagePortSetInvalidationCallBack)
        def invalidate(port, info):
            global didInvalidate
            didInvalidate = True

        CoreFoundation.CFMessagePortSetInvalidationCallBack(port, invalidate)
        cb = CoreFoundation.CFMessagePortGetInvalidationCallBack(port)

        # XXX: Without writing a custom wrapper we cannot guarantee this
        # self.assertIs(cb, invalidate)
        cb(None, None)
        self.assertIs(didInvalidate, True)
        didInvalidate = False

        rls = CoreFoundation.CFMessagePortCreateRunLoopSource(None, port, 0)
        self.assertIsInstance(rls, CoreFoundation.CFRunLoopSourceRef)
        self.assertResultIsBOOL(CoreFoundation.CFMessagePortIsValid)
        self.assertTrue(CoreFoundation.CFMessagePortIsValid(port))
        CoreFoundation.CFMessagePortInvalidate(port)
        self.assertFalse(CoreFoundation.CFMessagePortIsValid(port))
        self.assertTrue(didInvalidate)

    @min_os_level("10.5")
    def testSending(self):
        curloop = CoreFoundation.CFRunLoopGetCurrent()
        context = []

        def callout(port, messageid, data, info):
            info.append((port, messageid, data))
            return b"hello world"

        port, shouldFree = CoreFoundation.CFMessagePortCreateLocal(
            None, "pyobjc.test", callout, context, None
        )
        self.assertIsInstance(port, CoreFoundation.CFMessagePortRef)

        self.assertArgIsOut(CoreFoundation.CFMessagePortSendRequest, 6)
        rls = CoreFoundation.CFMessagePortCreateRunLoopSource(None, port, 0)
        CoreFoundation.CFRunLoopAddSource(
            curloop, rls, CoreFoundation.kCFRunLoopCommonModes
        )

        try:
            cli = CoreFoundation.CFMessagePortCreateRemote(None, "pyobjc.test")
            self.assertIsInstance(cli, CoreFoundation.CFMessagePortRef)
            self.assertIsNot(rls, None)
            if 0:
                err, data = CoreFoundation.CFMessagePortSendRequest(
                    cli,
                    99,
                    b"message",
                    1.0,
                    1.0,
                    CoreFoundation.kCFRunLoopDefaultMode,
                    None,
                )
                self.assertEqual(err, 0)
                self.assertEqual(data, b"hello world")

        finally:
            CoreFoundation.CFRunLoopRemoveSource(
                curloop, rls, CoreFoundation.kCFRunLoopCommonModes
            )



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSExtensionContext(TestCase):
    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertArgIsBlock(
            Foundation.NSExtensionContext.completeRequestReturningItems_completionHandler_,
            1,
            b"vZ",
        )
        self.assertArgIsBlock(
            Foundation.NSExtensionContext.openURL_completionHandler_, 1, b"vZ"
        )

    @min_os_level("10.10")
    def testConstant10_10(self):
        self.assertIsInstance(Foundation.NSExtensionItemsAndErrorsKey, str)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSMetaDataAttributes(TestCase):
    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertIsInstance(Foundation.NSMetadataItemFSNameKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemDisplayNameKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemURLKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemPathKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemFSSizeKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemFSCreationDateKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemFSContentChangeDateKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemIsUbiquitousKey, str)
        self.assertIsInstance(
            Foundation.NSMetadataUbiquitousItemHasUnresolvedConflictsKey, str
        )
        self.assertIsInstance(Foundation.NSMetadataUbiquitousItemIsDownloadedKey, str)
        self.assertIsInstance(Foundation.NSMetadataUbiquitousItemIsDownloadingKey, str)
        self.assertIsInstance(Foundation.NSMetadataUbiquitousItemIsUploadedKey, str)
        self.assertIsInstance(Foundation.NSMetadataUbiquitousItemIsUploadingKey, str)
        self.assertIsInstance(
            Foundation.NSMetadataUbiquitousItemPercentDownloadedKey, str
        )
        self.assertIsInstance(
            Foundation.NSMetadataUbiquitousItemPercentUploadedKey, str
        )

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertIsInstance(Foundation.NSMetadataItemContentTypeKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemContentTypeTreeKey, str)
        self.assertIsInstance(
            Foundation.NSMetadataUbiquitousItemDownloadingStatusKey, str
        )
        self.assertIsInstance(
            Foundation.NSMetadataUbiquitousItemDownloadingStatusDownloaded, str
        )
        self.assertIsInstance(
            Foundation.NSMetadataUbiquitousItemDownloadingStatusCurrent, str
        )
        self.assertIsInstance(
            Foundation.NSMetadataUbiquitousItemDownloadingErrorKey, str
        )
        self.assertIsInstance(Foundation.NSMetadataUbiquitousItemUploadingErrorKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemAttributeChangeDateKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemKeywordsKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemTitleKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemAuthorsKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemEditorsKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemParticipantsKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemProjectsKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemDownloadedDateKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemWhereFromsKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemCommentKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemCopyrightKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemLastUsedDateKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemContentCreationDateKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemContentModificationDateKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemDateAddedKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemDurationSecondsKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemContactKeywordsKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemVersionKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemPixelHeightKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemPixelWidthKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemPixelCountKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemColorSpaceKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemBitsPerSampleKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemFlashOnOffKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemFocalLengthKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemAcquisitionMakeKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemAcquisitionModelKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemISOSpeedKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemOrientationKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemLayerNamesKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemWhiteBalanceKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemApertureKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemProfileNameKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemResolutionWidthDPIKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemResolutionHeightDPIKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemExposureModeKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemExposureTimeSecondsKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemEXIFVersionKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemCameraOwnerKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemFocalLength35mmKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemLensModelKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemEXIFGPSVersionKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemAltitudeKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemLatitudeKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemLongitudeKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemSpeedKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemTimestampKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemGPSTrackKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemImageDirectionKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemGPSStatusKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemGPSMeasureModeKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemGPSDOPKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemGPSMapDatumKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemGPSDestLatitudeKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemGPSDestLongitudeKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemGPSDestBearingKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemGPSProcessingMethodKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemGPSAreaInformationKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemGPSDateStampKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemGPSDifferentalKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemCodecsKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemMediaTypesKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemStreamableKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemTotalBitRateKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemVideoBitRateKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemAudioBitRateKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemDeliveryTypeKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemAlbumKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemHasAlphaChannelKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemRedEyeOnOffKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemMeteringModeKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemMaxApertureKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemFNumberKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemExposureProgramKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemExposureTimeStringKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemHeadlineKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemInstructionsKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemCityKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemStateOrProvinceKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemCountryKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemTextContentKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemAudioSampleRateKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemAudioChannelCountKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemTempoKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemKeySignatureKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemTimeSignatureKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemAudioEncodingApplicationKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemComposerKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemLyricistKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemAudioTrackNumberKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemRecordingDateKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemMusicalGenreKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemIsGeneralMIDISequenceKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemRecordingYearKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemOrganizationsKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemLanguagesKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemRightsKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemPublishersKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemContributorsKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemCoverageKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemSubjectKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemThemeKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemDescriptionKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemIdentifierKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemAudiencesKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemNumberOfPagesKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemPageWidthKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemPageHeightKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemSecurityMethodKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemCreatorKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemEncodingApplicationsKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemDueDateKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemStarRatingKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemPhoneNumbersKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemEmailAddressesKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemInstantMessageAddressesKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemKindKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemRecipientsKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemFinderCommentKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemFontsKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemAppleLoopsRootKeyKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemAppleLoopsKeyFilterTypeKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemAppleLoopsLoopModeKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemAppleLoopDescriptorsKey, str)
        self.assertIsInstance(
            Foundation.NSMetadataItemMusicalInstrumentCategoryKey, str
        )
        self.assertIsInstance(Foundation.NSMetadataItemMusicalInstrumentNameKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemCFBundleIdentifierKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemInformationKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemDirectorKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemProducerKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemGenreKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemPerformersKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemOriginalFormatKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemOriginalSourceKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemAuthorEmailAddressesKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemRecipientEmailAddressesKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemAuthorAddressesKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemRecipientAddressesKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemIsLikelyJunkKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemExecutableArchitecturesKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemExecutablePlatformKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemApplicationCategoriesKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemIsApplicationManagedKey, str)

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(
            Foundation.NSMetadataUbiquitousItemDownloadRequestedKey, str
        )
        self.assertIsInstance(
            Foundation.NSMetadataUbiquitousItemIsExternalDocumentKey, str
        )
        self.assertIsInstance(
            Foundation.NSMetadataUbiquitousItemContainerDisplayNameKey, str
        )
        self.assertIsInstance(
            Foundation.NSMetadataUbiquitousItemURLInLocalContainerKey, str
        )

    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertIsInstance(
            Foundation.NSMetadataUbiquitousSharedItemCurrentUserRoleKey, str
        )
        self.assertIsInstance(
            Foundation.NSMetadataUbiquitousSharedItemCurrentUserPermissionsKey, str
        )
        self.assertIsInstance(
            Foundation.NSMetadataUbiquitousSharedItemOwnerNameComponentsKey, str
        )
        self.assertIsInstance(
            Foundation.NSMetadataUbiquitousSharedItemMostRecentEditorNameComponentsKey,
            str,
        )
        self.assertIsInstance(Foundation.NSMetadataUbiquitousSharedItemRoleOwner, str)
        self.assertIsInstance(
            Foundation.NSMetadataUbiquitousSharedItemRoleParticipant, str
        )
        self.assertIsInstance(
            Foundation.NSMetadataUbiquitousSharedItemPermissionsReadOnly, str
        )
        self.assertIsInstance(
            Foundation.NSMetadataUbiquitousSharedItemPermissionsReadWrite, str
        )
